#!/usr/bin/python
# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
#
from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Naval Patel (@navalkp)
  - Prashant Bhosale (@prabhosa)
module: lxca_cmms
short_description: Custom module for lxca cmms inventory utility
description:
  - This module returns/displays a inventory details of cmms.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  uuid:
    description: UUID of device, this is string with length greater than 16.
    type: str

  command_options:
    description: Options to filter nodes information.
    default: cmms
    choices:
      - cmms
      - cmms_by_uuid
      - cmms_by_chassis_uuid
    type: str

  chassis:
    description: UUID of chassis, this is string with length greater than 16.
    type: str

extends_documentation_fragment:
  - community.general._lxca_common
  - community.general._attributes
"""

EXAMPLES = r"""
# get all cmms info
- name: Get nodes data from LXCA
  community.general.lxca_cmms:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"

# get specific cmms info by uuid
- name: Get nodes data from LXCA
  community.general.lxca_cmms:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    uuid: "3C737AA5E31640CE949B10C129A8B01F"
    command_options: cmms_by_uuid

# get specific cmms info by chassis uuid
- name: Get nodes data from LXCA
  community.general.lxca_cmms:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    chassis: "3C737AA5E31640CE949B10C129A8B01F"
    command_options: cmms_by_chassis_uuid
"""

RETURN = r"""
result:
  description: Cmms detail from lxca.
  returned: success
  type: dict
  sample:
    cmmList:
      - machineType: ''
        model: ''
        type: 'CMM'
        uuid: '118D2C88C8FD11E4947B6EAE8B4BDCDF'
          # bunch of properties
      - machineType: ''
        model: ''
        type: 'CMM'
        uuid: '223D2C88C8FD11E4947B6EAE8B4BDCDF'
          # bunch of properties
        # Multiple cmms details
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lxca_common import (
    LXCA_COMMON_ARGS,
    connection_object,
    has_pylxca,
)

try:
    from pylxca import cmms
except ImportError:
    pass


UUID_REQUIRED = "UUID of device is required for cmms_by_uuid command."
CHASSIS_UUID_REQUIRED = "UUID of chassis is required for cmms_by_chassis_uuid command."


def _cmms(module, lxca_con):
    return cmms(lxca_con)


def _cmms_by_uuid(module, lxca_con):
    if not module.params["uuid"]:
        module.fail_json(msg=UUID_REQUIRED)
    return cmms(lxca_con, module.params["uuid"])


def _cmms_by_chassis_uuid(module, lxca_con):
    if not module.params["chassis"]:
        module.fail_json(msg=CHASSIS_UUID_REQUIRED)
    return cmms(lxca_con, chassis=module.params["chassis"])


def setup_module_object():
    """
    this function merge argument spec and create ansible module object
    :return:
    """
    args_spec = dict(LXCA_COMMON_ARGS)
    args_spec.update(INPUT_ARG_SPEC)
    module = AnsibleModule(argument_spec=args_spec, supports_check_mode=False)

    return module


FUNC_DICT = {
    "cmms": _cmms,
    "cmms_by_uuid": _cmms_by_uuid,
    "cmms_by_chassis_uuid": _cmms_by_chassis_uuid,
}


INPUT_ARG_SPEC = dict(
    command_options=dict(default="cmms", choices=["cmms", "cmms_by_uuid", "cmms_by_chassis_uuid"]),
    uuid=dict(),
    chassis=dict(),
)


def execute_module(module):
    """
    This function invoke commands
    :param module: Ansible module object
    """
    try:
        with connection_object(module) as lxca_con:
            result = FUNC_DICT[module.params["command_options"]](module, lxca_con)
            module.exit_json(changed=False, msg=f"Success {module.params['command_options']} result", result=result)
    except Exception as exception:
        error_msg = "; ".join((e) for e in exception.args)
        module.fail_json(msg=error_msg, exception=traceback.format_exc())


def main():
    module = setup_module_object()
    has_pylxca(module)
    execute_module(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_snapshot_info
version_added: "2.7.0"
short_description: Get the Azure Snapshot instance facts
description:
    - Get or list instance facts of Azure Snapshot.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    name:
        description:
            - Resource name.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get the snapshot instance by name
  azure_rm_snapshot_info:
    resource_group: myResourceGroup
    name: mySnapshot

- name: List all snapshots by resource group and filter by tags
  azure_rm_snapshot_info:
    resource_group: myResourceGroup
    name: mySnapshot
'''

RETURN = '''
state:
    description:
        - The description of the snapshot instance facts.
    type: complex
    returned: always
    contains:
        id:
            description:
                - The Snapshot instance ID.
            type: str
            returned: always
            sample: /subscription/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Compute/snapshots/mySnapshot01
        location:
            description:
                - The Snapshot instance location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The Snapshot instance name.
            type: str
            returned: always
            sample: mySnapshot01
        properties:
            description:
                - The properties of the snapshot instance.
            type: dict
            returned: always
            sample: {
                    "creationData": {
                        "createOption": "Import",
                        "sourceUri": "https://vmforimagerpfx01.blob.core.windows.net/vhds/vmforimagerpfx01.vhd"
                    },
                    "diskSizeBytes": 32213303296,
                    "diskSizeGB": 30,
                    "diskState": "Unattached",
                    "encryption": {
                        "type": "EncryptionAtRestWithPlatformKey"
                    },
                    "incremental": false,
                    "networkAccessPolicy": "AllowAll",
                    "provisioningState": "Succeeded",
                    "publicNetworkAccess": "Enabled",
                    "timeCreated": "2024-08-01T07:08:32.1635314+00:00",
                    "uniqueId": "85607f7a-ce44-40bd-a523-a7e2a10d72c3"
                   }
        sku:
            description:
                - The snapshots sku option.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - The Sku name.
                    type: str
                    returned: always
                    sample: Standard_LRS
                tier:
                    description:
                        - The Sku tier.
                    type: str
                    returned: always
                    sample: Standard
        tags:
            description:
                - Resource tags.
            type: dict
            returned: always
            sample: {'key1': 'value1'}
        type:
            description:
                - Resource type.
            type: str
            returned: always
            sample: Microsoft.Compute/snapshots
'''

import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class AzureRMSnapshotsInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )

        self.resource_group = None
        self.name = None
        self.tags = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.url = None

        self.query_parameters = {}
        self.status_code = [200, 201, 202]
        self.query_parameters['api-version'] = '2022-03-02'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureRMSnapshotsInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                   supports_check_mode=True,
                                                   facts_module=True,
                                                   supports_tags=False)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if self.name is not None:
            self.results['state'] = self.get()
        else:
            self.results['state'] = self.list_by_resourcegroup()

        return self.results

    def get(self):
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Compute' +
                    '/snapshots' +
                    '/{{ snapshot_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ snapshot_name }}', self.name)
        response = None
        try:
            response = self.mgmt_client.query(url=self.url,
                                              method='GET',
                                              query_parameters=self.query_parameters,
                                              header_parameters=self.header_parameters,
                                              body=None,
                                              expected_status_codes=self.status_code,
                                              polling_timeout=600,
                                              polling_interval=30)
            response = json.loads(response.body())
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Did not find the Snapshot instance.')
        if response and self.has_tags(response.get('tags'), self.tags):
            return [response]
        else:
            return []

    def list_by_resourcegroup(self):
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Compute' +
                    '/snapshots')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        response = None
        results = []

        try:
            response = self.mgmt_client.query(url=self.url,
                                              method='GET',
                                              query_parameters=self.query_parameters,
                                              header_parameters=self.header_parameters,
                                              body=None,
                                              expected_status_codes=self.status_code,
                                              polling_timeout=600,
                                              polling_interval=30)
            response = json.loads(response.body())
            self.log("Response : {0}".format(response))
            # self.log("Snapshot instance : {0} found".format(response.name))
        except Exception as e:
            self.log('Did not find the Snapshot instance.')
        for item in response['value']:
            if self.has_tags(item.get('tags'), self.tags):
                results.append(item)

        return results


def main():
    AzureRMSnapshotsInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
flatten a complex object to dot bracket notation
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from collections.abc import Mapping, MutableMapping


def to_paths(var, prepend, wantlist):
    if prepend:
        var = {prepend: var}

    def flatten(data, name="", out=None):
        if out is None:
            out = {}
        if isinstance(data, (dict, Mapping, MutableMapping)):
            if data:
                for key, val in data.items():
                    if name:
                        if re.match("^[a-zA-Z_][a-zA-Z0-9_]*$", key):
                            nname = name + ".{key}".format(key=key)
                        else:
                            nname = name + "['{key}']".format(key=key)
                    else:
                        nname = key
                    flatten(val, nname, out)
            elif name:
                out[name] = {}
            else:
                out = {}
        elif isinstance(data, list):
            if data:
                for idx, val in enumerate(data):
                    flatten(val, "{name}[{idx}]".format(name=name, idx=idx), out)
            elif name:
                out[name] = []
            else:
                out = []
        else:
            out[name] = data
        return out

    out = flatten(var)
    if wantlist:
        return [out]
    return out



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard files documentation fragment
    DOCUMENTATION = r"""
attributes:
  idempotent:
    support: full
options:
  zone_name:
    description:
      - The DNS zone to query.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
    type: str
    aliases:
      - zone
  zone_id:
    description:
      - The ID of the DNS zone to query.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
    version_added: 0.2.0
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r"""
name: meraki
author:
  - Nilashish Chakraborty (@NilashishC)
short_description: Ansible dynamic inventory plugin for Cisco Meraki devices.
requirements:
  - meraki
extends_documentation_fragment:
  - constructed
description:
  - Build inventories using the Cisco Meraki API.
  - Uses a YAML configuration file cisco_meraki.[yml|yaml].
options:
  meraki_org_id:
    description:
      - The organization ID to fetch the networks and devices from.
    type: str
    required: true
  meraki_api_key:
    description:
      - meraki_api_key (string), API key generated in dashboard; can also be set as an environment variable MERAKI_DASHBOARD_API_KEY
    type: str
  meraki_base_url:
    description:
        - meraki_base_url (string), preceding all endpoint resources
    type: str
    default: https://api.meraki.com/api/v1
  meraki_single_request_timeout:
    description:
      - meraki_single_request_timeout (integer), maximum number of seconds for each API call
    type: int
    default: 60
  meraki_certificate_path:
    description:
      - meraki_certificate_path (string), path for TLS/SSL certificate verification if behind local proxy
    type: str
    default: ''
  meraki_requests_proxy:
    description:
      - meraki_requests_proxy (string), proxy server and port, if needed, for HTTPS
    type: str
    default: ''
  meraki_wait_on_rate_limit:
    description:
      - meraki_wait_on_rate_limit (boolean), retry if 429 rate limit error encountered?
    type: bool
    default: true
  meraki_nginx_429_retry_wait_time:
    description:
      - meraki_nginx_429_retry_wait_time (integer), Nginx 429 retry wait time
    type: int
    default: 60
  meraki_action_batch_retry_wait_time:
    description:
      - meraki_action_batch_retry_wait_time (integer), action batch concurrency error retry wait time
    type: int
    default: 60
  meraki_retry_4xx_error:
    description:
      - meraki_retry_4xx_error (boolean), retry if encountering other 4XX error (besides 429)?
    type: bool
    default: false
  meraki_retry_4xx_error_wait_time:
    description:
      - meraki_retry_4xx_error_wait_time (integer), other 4XX error retry wait time
    type: int
    default: 60
  meraki_maximum_retries:
    description:
      - meraki_maximum_retries (integer), retry up to this many times when encountering 429s or other server-side errors
    type: int
    default: 2
  meraki_output_log:
    description:
      - meraki_output_log (boolean), create an output log file?
    type: bool
    default: true
  meraki_log_file_prefix:
    description:
      - meraki_log_file_prefix (string), log file name appended with date and timestamp
    type: str
    default: meraki_api_
  meraki_log_path:
    description:
      - log_path (string), path to output log; by default, working directory of script if not specified
    type: str
    default: ''
  meraki_print_console:
    description:
      - meraki_print_console (boolean), print logging output to console?
    type: bool
    default: true
  meraki_suppress_logging:
    description:
      - meraki_suppress_logging (boolean), disable all logging? you're on your own then!
    type: bool
    default: false
  meraki_simulate:
    description:
      - meraki_simulate (boolean), simulate POST/PUT/DELETE calls to prevent changes?
    type: bool
    default: false
  meraki_be_geo_id:
    description:
      - meraki_be_geo_id (string), optional partner identifier for API usage tracking; can also be set as an environment variable BE_GEO_ID
    type: str
    default: ''
  meraki_use_iterator_for_get_pages:
    description:
      - meraki_use_iterator_for_get_pages (boolean), list* methods will return an iterator with each object instead of a complete list with all items
    type: bool
    default: false
  meraki_inherit_logging_config:
    description:
      - meraki_inherit_logging_config (boolean), Inherits your own logger instance
    type: bool
    default: false
"""

EXAMPLES = r"""
# cisco_meraki.yml
---
plugin: cisco.meraki.meraki
meraki_api_key: "<enter Meraki API key or set the MERAKI_DASHBOARD_API_KEY env var>"
meraki_org_id: "<enter Meraki Org ID>"
keyed_groups:
  # group devices based on network ID
  - prefix: meraki_network_id
    key: network_id
  # group devices based on network name
  - prefix: meraki_network
    key: network
  # group devices based on device type
  - prefix: meraki_device_type
    key: device_type
  # group devices based on meraki device tag
  - prefix: meraki_tag
    key: tags
"""

from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
)
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.module_utils.common.arg_spec import ArgumentSpecValidator
from ansible.errors import AnsibleError


meraki_argument_spec = meraki_argument_spec()
meraki_argument_spec.update(
    dict(meraki_org_id=dict(type="str", required=True)))


class InventoryModule(BaseInventoryPlugin, Constructable):
    NAME = "cisco.meraki.meraki"

    def verify_file(self, path):
        """return true/false if this is possibly a valid file for this plugin to consume"""
        valid = False
        if super(InventoryModule, self).verify_file(path):
            # base class verifies that file exists and is readable by current user
            if path.endswith(("cisco_meraki.yaml", "cisco_meraki.yml")):
                valid = True
        return valid

    def _build_network_map(self, dashboard, org_id):
        """Build a dictionary mapping network ID to network names."""
        network_map = {}
        networks = dashboard.exec_meraki(
            family="organizations",
            function="getOrganizationNetworks",
            params={"organizationId": org_id, "total_pages": "all"}
        )
        for network in networks:
            network_map[network["id"]] = to_text(network.get("name", ""))

        return network_map

    def _validate_argspec(self, parameters):
        "Validate the inventory plugin argspec."
        argspec_validator = ArgumentSpecValidator(meraki_argument_spec)
        result = argspec_validator.validate(parameters)

        if result.error_messages:
            raise AnsibleError(
                f"Validation failed: {', '.join(result.error_messages)}")

        return result.validated_parameters

    def parse(self, inventory, loader, path, cache=True):
        """Talk to the Meraki API and build the inventory."""

        # call base method to ensure properties are available for use with other helper methods
        super(InventoryModule, self).parse(inventory, loader, path, cache)

        config = self._read_config_data(path)
        meraki_connect_config = {
            k: config[k] for k in config if k in meraki_argument_spec
        }

        validated_config = self._validate_argspec(
            parameters=meraki_connect_config)

        meraki_org_id = validated_config.pop("meraki_org_id")

        strict = self.get_option("strict")

        dashboard = MERAKI(params=validated_config)

        try:
            devices = dashboard.exec_meraki(
                family="organizations",
                function="getOrganizationDevices",
                params={"organizationId": meraki_org_id, "total_pages": "all"},
            )
            if devices:
                network_map = self._build_network_map(dashboard, meraki_org_id)

                for device in devices:
                    hostname = device.get("name")
                    if not hostname:
                        self.display.warning(
                            f"No name set for device with MAC {device['mac']}"
                        )
                        hostname = device["mac"]
                    self.inventory.add_host(hostname, group="all")

                    if device.get("networkId"):
                        self.inventory.set_variable(
                            hostname, "network_id", device["networkId"]
                        )
                        self.inventory.set_variable(
                            hostname, "network", network_map[device["networkId"]]
                        )
                    else:
                        self.display.vvvv(
                            f"Device {hostname} is not associated with a network."
                        )

                    if device.get("lanIp"):
                        self.inventory.set_variable(
                            hostname, "ansible_host", device["lanIp"]
                        )
                    else:
                        self.display.warning(
                            f"No lanIp found for device with {hostname}."
                            " The `ansible_host` variable will not be set for this host."
                        )

                    self.inventory.set_variable(
                        hostname, "device_type", device["model"]
                    )

                    self.inventory.set_variable(
                        hostname, "ansible_product_serial", device["serial"]
                    )
                    self.inventory.set_variable(
                        hostname, "macaddress", device["mac"])

                    # Add variables created by the user's Jinja2 expressions to the host
                    self._set_composite_vars(self.get_option(
                        "compose"), device, hostname, strict=True)

                    # Create user-defined groups using variables and Jinja2 conditionals
                    self._add_host_to_composed_groups(self.get_option(
                        "groups"), device, hostname, strict=strict)
                    # Add the host to the keyed groups
                    self._add_host_to_keyed_groups(
                        keys=self.get_option("keyed_groups"),
                        variables=device,
                        host=hostname,
                        strict=strict,
                    )
        except Exception as e:
            raise AnsibleError(
                f"Failed to get devices from Meraki API: {to_native(e)}")



# pylint: skip-file
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_acls fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible.module_utils._text import to_text
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.acls.acls import (
    AclsArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.acls import (
    AclsTemplate,
)


class AclsFacts(object):
    """The ios_acls fact class"""

    def __init__(self, module):
        self._module = module
        self.argument_spec = AclsArgs.argument_spec

    def get_acl_data(self, connection):
        # Removed the show access-list
        # Removed the show running-config | include ip(v6)* access-list|remark
        return connection.get("show running-config | section access-list")

    def get_acl_names(self, connection):
        # this information is required to scoop out the access lists which has no aces
        return connection.get("show access-lists | include access list")

    def populate_empty_acls(self, raw_acls, raw_acls_name):
        # this would update empty acls to the full acls entry
        if raw_acls and raw_acls_name:
            for aclnames, acldata in raw_acls_name.get("acls").items():
                if aclnames not in raw_acls.get("acls").keys():
                    if not raw_acls.get("acls"):
                        raw_acls["acls"] = {}
                    raw_acls["acls"][aclnames] = acldata
        elif raw_acls_name and not raw_acls:
            for aclnames, acldata in raw_acls_name.get("acls").items():
                if not raw_acls.get("acls"):
                    raw_acls["acls"] = {}
                raw_acls["acls"][aclnames] = acldata
        return raw_acls

    def sanitize_data(self, data):
        """removes matches or extra config info that is added on acl match"""
        re_data = ""
        remarks_idx = 0
        for da in data.split("\n"):
            if "match" in da:
                mod_da = re.sub(r"\([^()]*\)", "", da)
                re_data += mod_da[:-1] + "\n"
            elif re.match(r"\s*\d+\sremark.+", da, re.IGNORECASE) or re.match(
                r"\s*remark.+",
                da,
                re.IGNORECASE,
            ):
                remarks_idx += 1
                re_data += to_text(remarks_idx) + " " + da + "\n"
            else:
                re_data += da + "\n"
        return re_data

    IGMP_MAP = {
        "1": "host_query",
        "2": "host_report",
        "3": "dvmrp",
        "4": "pim",
        "5": "trace",
    }

    @staticmethod
    def _normalize_protocol_options(each_ace):
        if each_ace.get("protocol_number"):
            each_ace["protocol_options"] = {
                "protocol_number": each_ace.pop("protocol_number"),
            }
        protocol = each_ace.get("protocol")
        protocol_options = each_ace.get("protocol_options")
        if protocol_options is not None:
            if protocol == "tcp":
                if isinstance(protocol_options, str):
                    flag_dict = {flag: True for flag in protocol_options.split()}
                    each_ace["protocol_options"] = {"tcp": flag_dict}
                else:
                    each_ace.pop("protocol_options", None)
            elif protocol == "icmp":
                protocol_options = str(protocol_options).replace("-", "_")
                each_ace["protocol_options"] = {"icmp": {protocol_options: True}}
            elif protocol == "igmp":
                protocol_options = str(protocol_options).replace("-", "_")
                mapping = AclsFacts.IGMP_MAP.get(protocol_options, protocol_options)
                each_ace["protocol_options"] = {"igmp": {mapping: True}}

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for acls
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        namedata = ""

        if not data:
            data = self.get_acl_data(connection)
            namedata = self.get_acl_names(connection)

        if data:
            data = self.sanitize_data(data)

        # parse main information
        templateObjMain = NetworkTemplate(lines=data.splitlines(), tmplt=AclsTemplate())
        raw_acls = templateObjMain.parse()

        if namedata:
            # parse just names to update empty acls
            templateObjName = NetworkTemplate(
                lines=namedata.splitlines(),
                tmplt=AclsTemplate(),
            )
            raw_acl_names = templateObjName.parse()
            raw_acls = self.populate_empty_acls(raw_acls, raw_acl_names)

        temp_v4 = []
        temp_v6 = []

        if raw_acls.get("acls"):
            for k, v in raw_acls.get("acls").items():
                if v.get("afi") == "ipv4" and v.get("acl_type") in [
                    "standard",
                    "extended",
                ]:
                    del v["afi"]
                    temp_v4.append(v)
                elif v.get("afi") == "ipv6":
                    del v["afi"]
                    temp_v6.append(v)

            temp_v4 = sorted(temp_v4, key=lambda i: str(i["name"]))
            temp_v6 = sorted(temp_v6, key=lambda i: str(i["name"]))

            def factor_source_dest(ace, typ):
                temp = ace.get(typ, {})
                # TODO complete the ipv6 delta logic
                if temp.get("address"):
                    ace[typ]["address"] = temp.get("address")
                    ace[typ]["wildcard_bits"] = temp.get("wildcard_bits")

            def process_protocol_options(each):
                for each_ace in each.get("aces"):
                    if each.get("acl_type") == "standard":
                        if len(each_ace.get("source", {})) == 1 and each_ace.get(
                            "source",
                            {},
                        ).get(
                            "address",
                        ):
                            each_ace["source"]["host"] = each_ace["source"].pop(
                                "address",
                            )
                        if each_ace.get("source", {}).get("address"):
                            addr = each_ace.get("source", {}).get("address")
                            if addr[-1] == ",":
                                each_ace["source"]["address"] = addr[:-1]
                    else:  # for extended acl
                        if each_ace.get("source", {}):
                            factor_source_dest(each_ace, "source")
                        if each_ace.get("destination", {}):
                            factor_source_dest(each_ace, "destination")

                    AclsFacts._normalize_protocol_options(each_ace)

            def collect_remarks(aces):
                """makes remarks list per ace"""
                ace_entry = []
                ace_rem = []
                rem = {}
                # every remarks is one list item which has a sequence number
                # every ace remark is preserved and ordered
                # at the end of each sequence it is flushed to a ace entry
                for i in aces:
                    # i here denotes an ace, which would be populated with remarks entries
                    if i.get("is_remark_for"):
                        if not rem.get(i.get("is_remark_for")):
                            rem[i.get("is_remark_for")] = {"remarks": []}
                            rem[i.get("is_remark_for")]["remarks"].append(
                                i.get("the_remark"),
                            )
                        else:
                            rem[i.get("is_remark_for")]["remarks"].append(
                                i.get("the_remark"),
                            )
                    else:
                        if rem:
                            if rem.get(i.get("sequence")):
                                ace_rem = rem.pop(i.get("sequence"))
                                i["remarks"] = ace_rem.get("remarks")
                        ace_entry.append(i)

                if rem:  # pending remarks
                    for pending_rem_seq, pending_rem_val in rem.items():
                        # there can be ace entry with just a remarks and no ace actually
                        # 10 remarks I am a remarks
                        # 20 ..... so onn
                        if pending_rem_seq != "remark":
                            ace_entry.append(
                                {
                                    "sequence": pending_rem_seq,
                                    "remarks": pending_rem_val.get("remarks"),
                                },
                            )
                        else:
                            # this handles the classic set of remarks at the end, which is not tied to
                            # any sequence number
                            pending_rem = rem.get("remark", {})
                            ace_entry.append({"remarks": pending_rem.get("remarks")})
                return ace_entry

            for each in temp_v4:
                if each.get("aces"):
                    # handling remarks for each ace entry
                    each["aces"] = collect_remarks(each.get("aces"))
                    process_protocol_options(each)

            for each in temp_v6:
                if each.get("aces"):
                    each["aces"] = collect_remarks(each.get("aces"))
                    process_protocol_options(each)

        objs = []
        if temp_v4:
            objs.append({"afi": "ipv4", "acls": temp_v4})
        if temp_v6:
            objs.append({"afi": "ipv6", "acls": temp_v6})

        facts = {}
        if objs:
            facts["acls"] = []
            params = utils.validate_config(self.argument_spec, {"config": objs})
            for cfg in params["config"]:
                facts["acls"].append(utils.remove_empties(cfg))
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_show_logs
short_description: Showing logs according to the given filter.
description:
  - Showing logs according to the given filter.
  - All operations are performed over Web Services API.
  - Available from R80.40 JHF management version.
version_added: "2.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  new_query:
    description:
      - Running a new query.
    type: dict
    suboptions:
      filter:
        description:
          - The filter as entered in SmartConsole/SmartView.
        type: str
      time_frame:
        description:
          - Specify the time frame to query logs.
        type: str
        choices: ['last-7-days', 'last-hour', 'today', 'last-24-hours', 'yesterday', 'this-week', 'this-month', 'last-30-days', 'all-time', 'custom']
      custom_start:
        description:
          - This option is only applicable when using the custom time-frame option.
        type: str
      custom_end:
        description:
          - This option is only applicable when using the custom time-frame option.
        type: str
      max_logs_per_request:
        description:
          - Limit the number of logs to be retrieved.
        type: int
      top:
        description:
          - Top results configuration.
        type: dict
        suboptions:
          field:
            description:
              - The field on which the top command is executed.
            type: str
            choices: ['sources', 'destinations', 'services', 'actions', 'blades' , 'origins', 'users', 'applications']
          count:
            description:
              - The number of results to retrieve.
            type: int
      type:
        description:
          - Type of logs to return.
        type: str
        choices: ['logs', 'audit']
      log_servers:
        description:
          - List of IP's of logs servers to query.
        type: list
        elements: str
  query_id:
    description:
      - Get the next page of last run query with specified limit.
    type: str
  ignore_warnings:
    description:
      - Ignore warnings if exist.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: show-logs
  cp_mgmt_show_logs:
    new_query:
      filter: blade:"Threat Emulation"
      max_logs_per_request: '2'
      time_frame: today
"""

RETURN = """
cp_mgmt_show_logs:
  description: The checkpoint show-logs output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        new_query=dict(
            type="dict",
            options=dict(
                filter=dict(type="str"),
                time_frame=dict(
                    type="str",
                    choices=[
                        "last-7-days",
                        "last-hour",
                        "today",
                        "last-24-hours",
                        "yesterday",
                        "this-week",
                        "this-month",
                        "last-30-days",
                        "all-time",
                        "custom",
                    ],
                ),
                custom_start=dict(type="str"),
                custom_end=dict(type="str"),
                max_logs_per_request=dict(type="int"),
                top=dict(
                    type="dict",
                    options=dict(
                        field=dict(
                            type="str",
                            choices=[
                                "sources",
                                "destinations",
                                "services",
                                "actions",
                                "blades",
                                "origins",
                                "users",
                                "applications",
                            ],
                        ),
                        count=dict(type="int"),
                    ),
                ),
                type=dict(type="str", choices=["logs", "audit"]),
                log_servers=dict(type="list", elements="str"),
            ),
        ),
        query_id=dict(type="str"),
        ignore_warnings=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "show-logs"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: github_secrets_info
short_description: List GitHub repository or organization secrets
description:
  - List secrets in a GitHub repository or organization.
author:
  - Thomas Sjögren (@konstruktoid)
version_added: '12.5.0'
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  organization:
    description:
      - The GitHub username or organization name.
    type: str
    required: true
    aliases: ["org", "username"]
  repository:
    description:
      - The name of the repository.
      - If not provided, the listing will be at organization level.
    type: str
    aliases: ["repo"]
  api_url:
    description:
      - The base URL for the GitHub API.
    type: str
    default: "https://api.github.com"
  token:
    description:
      - The GitHub token used for authentication.
    type: str
    required: true
"""

EXAMPLES = r"""
- name: List Github secret
  community.general.github_secrets_info:
    token: "{{ lookup('ansible.builtin.env', 'GITHUB_TOKEN') }}"
    repository: "ansible"
    organization: "ansible"
"""

RETURN = r"""
secrets:
  description: The list of currently existing secrets.
  type: list
  elements: dict
  returned: success
  sample: [
    {
      "created_at": "2026-01-11T23:19:00Z",
      "name": "ANSIBLE",
      "updated_at": "2026-02-15T22:18:16Z"
    },
  ]
  contains:
    name:
      description: The name of the secret.
      type: str
    created_at:
      description: The date and time when the secret was created.
      type: str
    updated_at:
      description: The date and time when the secret was last updated.
      type: str
"""

import json
import typing as t
from http import HTTPStatus

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils import _deps as deps


def list_secrets(
    module: AnsibleModule,
    api_url: str,
    headers: dict[str, str],
    organization: str,
    repository: str,
) -> dict[str, list]:
    url = (
        f"{api_url}/repos/{organization}/{repository}/actions/secrets"
        if repository
        else f"{api_url}/orgs/{organization}/actions/secrets"
    )

    resp, info = fetch_url(module, url, headers=headers, method="GET")

    if info["status"] == HTTPStatus.OK:
        body = resp.read()
        return {"secrets": json.loads(body).get("secrets", [])}
    elif info["status"] == HTTPStatus.NOT_FOUND:
        return {
            "secrets": [],
        }
    else:
        module.fail_json(msg=f"Failed to list secrets: {info}")


def main() -> None:
    """Ansible module entry point."""
    argument_spec = {
        "organization": {
            "type": "str",
            "aliases": ["org", "username"],
            "required": True,
        },
        "repository": {"type": "str", "aliases": ["repo"]},
        "api_url": {"type": "str", "default": "https://api.github.com"},
        "token": {"type": "str", "required": True, "no_log": True},
    }

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    deps.validate(module)

    organization: str = module.params["organization"]
    repository: str = module.params["repository"]
    api_url: str = module.params["api_url"]
    token: str = module.params["token"]

    result: dict[str, t.Any] = {}

    headers = {
        "Accept": "application/vnd.github+json",
        "Authorization": f"Bearer {token}",
        "Content-Type": "application/json",
    }

    secrets = list_secrets(module, api_url, headers, organization, repository)

    result["changed"] = False
    result.update(
        secrets=secrets["secrets"],
    )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Andrew Klychkov (@Andersson007) <andrew.a.klychkov@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: clickhouse_client

short_description: Execute queries in a ClickHouse database using the clickhouse-driver Client interface

description:
  - Execute arbitrary queries in a ClickHouse database using the
    L(clickhouse-driver,https://clickhouse-driver.readthedocs.io/en/latest) Client interface.
  - Always returns that the state changed.

version_added: '0.1.0'

author:
  - Andrew Klychkov (@Andersson007)
  - Aleks Vagachev (@aleksvagachev)

extends_documentation_fragment:
  - community.clickhouse.client_inst_opts

notes:
  - Does not support C(check_mode).

options:
  execute:
    description:
      - A query to the V(Client.execute(\)) method.
    type: str
    required: true

  execute_kwargs:
    description:
      - All additional keyword arguments you want to pass to
        the V(Client.execute(\)) method. For example, you can pass
        substitution parameters for the query you pass
        through the I(execute) argument.
    type: dict
    default: {}

  set_settings:
    description:
      - The dict of settings that need to be set in the session before executing the request.
    type: dict
    default: {}
    version_added: '0.5.0'
attributes:
  idempotent:
    support: none
    description:
      - Module always will return changed=true.
  check_mode:
    support: none
    description:
      - Module executes arbitary query. Check mode not supported.
'''

EXAMPLES = r'''
- name: Query DB using non-default user & DB to connect to
  register: result
  community.clickhouse.clickhouse_client:
    execute: SELECT * FROM my_table
    login_host: localhost
    login_user: alice
    login_db: foo
    login_password: my_password

- name: Print returned rows
  ansible.builtin.debug:
    var: result.result

- name: Create table
  register: result
  community.clickhouse.clickhouse_client:
    execute: CREATE TABLE test_table_1 (x String) ENGINE = Memory
    set_settings:
      flatten_nested: 0
      short_circuit_function_evaluation: 'disable'

- name: Insert into test table using named parameters
  register: result
  community.clickhouse.clickhouse_client:
    execute: "INSERT INTO test_table_1 (x) VALUES (%(a)s), (%(b)s), (%(c)s)"
    execute_kwargs:
      params:
        a: one
        b: two
        c: three

- name: Check the result
  ansible.builtin.assert:
    that:
      - result.statistics["processed_rows"] == 3
      - result.substituted_query == "INSERT INTO test_table_1 (x) VALUES ('one'), ('two'), ('three')"

- name: Check rows were inserted into test table
  register: result
  community.clickhouse.clickhouse_client:
    execute: "SELECT * FROM test_table_1"

- name: Check returned values
  ansible.builtin.assert:
    that:
    - result.result == [["one"], ["two"], ["three"]]
'''

RETURN = r'''
substituted_query:
  description:
  - Executed query with substituted arguments if any.
  returned: on success
  sample: SELECT * FROM test_table_1
  type: str
result:
  description:
  - Result returned by Client.execute().
  returned: on success
  type: list
  sample: [["one"], ["two"], ["three"]]
statistics:
  description:
  - Last executed query statistics retrieved from the
    L(last_query,https://clickhouse-driver.readthedocs.io/en/latest/features.html#query-execution-statistics)
    attribute.
  - Returned items depend on server version.
  returned: on success
  type: dict
'''
from decimal import Decimal

HAS_IPADDRESS = False
try:
    from ipaddress import IPv4Address, IPv6Address
    HAS_IPADDRESS = True
except ImportError:
    pass

from uuid import UUID

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.clickhouse.plugins.module_utils.clickhouse import (
    check_clickhouse_driver,
    client_common_argument_spec,
    get_main_conn_kwargs,
    connect_to_db_via_client,
    execute_query,
)


def is_uuid(value):
    """Checks if the value is valid UUID.

    Returns True if yes, False otherwise.
    """
    try:
        UUID(str(value))
        return True
    except ValueError:
        return False


def vals_to_supported(result):
    """Converts values of types unsupported by Ansible Core
    to supported.

    Ansible Core has a limited set of supported values.
    This is intentional, and the docs says such values
    have to be converted on the module's side.

    Add more values here if needed.
    """
    for idx_row, row in enumerate(result):
        for idx_val, val in enumerate(row):
            if is_uuid(val):
                # As tuple does not support change,
                # we need some conversion here
                row = replace_val_in_tuple(row, idx_val, str(val))
                result[idx_row] = row

            elif isinstance(val, Decimal):
                row = replace_val_in_tuple(row, idx_val, float(val))
                result[idx_row] = row

            elif isinstance(val, IPv4Address) or isinstance(val, IPv6Address):
                row = replace_val_in_tuple(row, idx_val, str(val))
                result[idx_row] = row

    return result


def replace_val_in_tuple(tup, idx, val):
    """Creates another tuple from a tuple substituting a value.

    Returns a new tuple.
    """
    tmp = list(tup)
    tmp[idx] = val
    return tuple(tmp)


def get_query_statistics(module, client):
    """Retrieve query statistics from the Client() object.

    Returns a dictionary with statistics.
    """
    statistics = {}
    try:
        # IMPORTANT: When adding new values, use hassattr() to check
        # if supported as below! This is important for compatibility reasons.
        if hasattr(client.last_query, 'elapsed'):
            statistics['elapsed'] = client.last_query.elapsed

        if not hasattr(client.last_query, 'progress'):
            return statistics

        if hasattr(client.last_query.progress, 'rows'):
            statistics['processed_rows'] = client.last_query.progress.rows

        if hasattr(client.last_query.progress, 'bytes'):
            statistics['processed_bytes'] = client.last_query.progress.bytes

        if hasattr(client.last_query.progress, 'total_rows'):
            statistics['total_rows'] = client.last_query.progress.total_rows

        if hasattr(client.last_query.progress, 'written_rows'):
            # As supported since driver version 0.1.3
            statistics['written_rows'] = client.last_query.progress.written_rows

        if hasattr(client.last_query.progress, 'written_bytes'):
            # As supported since driver version 0.1.3
            statistics['written_bytes'] = client.last_query.progress.written_bytes

        if hasattr(client.last_query.progress, 'elapsed_ns'):
            # As supported since driver version 0.2.7
            statistics['elapsed_ns'] = client.last_query.progress.elapsed_ns

    except Exception as e:
        module.fail_json(msg="Failed to retrieve statistics: %s" % to_native(e))

    return statistics


def get_substituted_query(module, client, query, execute_kwargs):
    """Substitute params in a query. If no params, just return the query as is.

    Returns a query with substituted params.
    """
    substituted_query = None
    if execute_kwargs.get("params"):
        try:
            substituted_query = client.substitute_params(query, execute_kwargs["params"],
                                                         context=client.connection.context)
        except Exception as e:
            module.fail_json(msg="Failed to substitute query params: %s" % to_native(e))
    else:
        # If no params were passed, no need to substitute.
        # Return query as is.
        substituted_query = query

    return substituted_query


def main():
    # Set up arguments.
    # If there are common arguments shared across several modules,
    # create the common_argument_spec() function under plugins/module_utils/*
    # and invoke here to return a dict with those arguments
    argument_spec = client_common_argument_spec()
    argument_spec.update(
        execute=dict(type='str', required=True),
        execute_kwargs=dict(type='dict', default={}),
        set_settings=dict(type='dict', default={})
    )

    # Instantiate an object of module class
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
    )

    # Assign passed options to variables
    client_kwargs = module.params['client_kwargs']
    query = module.params['execute']
    execute_kwargs = module.params['execute_kwargs']
    set_settings = module.params['set_settings']
    # The reason why these arguments are separate from client_kwargs
    # is that we need to protect some sensitive data like passwords passed
    # to the module from logging (see the arguments above with no_log=True);
    # Such data must be passed as module arguments (not nested deep in values).
    main_conn_kwargs = get_main_conn_kwargs(module)

    # Will fail if no driver informing the user
    check_clickhouse_driver(module)

    # There's no ipaddress package in Python 2
    if not HAS_IPADDRESS:
        msg = ("If you use Python 2 on your target host, "
               "make sure you have the py2-ipaddress Python package installed there to avoid "
               "crashes while querying tables containing columns of IPv4|6Address types.")
        module.warn(msg)

    # Connect to DB
    client = connect_to_db_via_client(module, main_conn_kwargs, client_kwargs)

    # Substitute query params if needed for future return
    substituted_query = get_substituted_query(module, client, query, execute_kwargs)

    # Execute query
    result = execute_query(module, client, query, execute_kwargs, set_settings)

    # Convert values not supported by ansible-core
    if result and isinstance(result, (list, tuple)):
        result = vals_to_supported(result)

    # Retreive statistics if present
    statistics = get_query_statistics(module, client)

    # Andersson007 doesn't see any way of checking if anything in DB was changed
    # or not by the query using the Client() interface
    # (considering all kinds of queries like DML, DDL, etc.)
    # It is safer and easier to always return changed=True.
    # Before changing this, think many times if you want to make
    # the code more complex and to support this complexity
    changed = True

    # Close connection
    client.disconnect_connection()

    # Users will get this in JSON output after execution
    kw = dict(
        changed=changed,
        result=result,
        substituted_query=substituted_query,
        statistics=statistics,
    )

    module.exit_json(**kw)


if __name__ == '__main__':
    main()



# Copyright (c) 2024 Vladimir Botka <vbotka@gmail.com>
# Copyright (c) 2024 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: replace_keys
short_description: Replace specific keys in a list of dictionaries
version_added: "9.1.0"
author:
  - Vladimir Botka (@vbotka)
  - Felix Fontein (@felixfontein)
description: This filter replaces specified keys in a provided list of dictionaries.
options:
  _input:
    description:
      - A list of dictionaries.
      - Top level keys must be strings.
    type: list
    elements: dictionary
    required: true
  target:
    description:
      - A list of dictionaries with attributes C(before) and C(after).
      - The value of O(target[].after) replaces key matching O(target[].before).
    type: list
    elements: dictionary
    required: true
    suboptions:
      before:
        description:
          - A key or key pattern to change.
          - The interpretation of O(target[].before) depends on O(matching_parameter).
          - For a key that matches multiple O(target[].before)s, the B(first) matching O(target[].after) is used.
        type: str
      after:
        description: A matching key change to.
        type: str
  matching_parameter:
    description: Specify the matching option of target keys.
    type: str
    default: equal
    choices:
      equal: Matches keys of exactly one of the O(target[].before) items.
      starts_with: Matches keys that start with one of the O(target[].before) items.
      ends_with: Matches keys that end with one of the O(target[].before) items.
      regex: Matches keys that match one of the regular expressions provided in O(target[].before).
"""

EXAMPLES = r"""
- l:
    - {k0_x0: A0, k1_x1: B0, k2_x2: [C0], k3_x3: foo}
    - {k0_x0: A1, k1_x1: B1, k2_x2: [C1], k3_x3: bar}

  # 1) By default, replace keys that are equal any of the attributes before.
- t:
    - {before: k0_x0, after: a0}
    - {before: k1_x1, after: a1}
  r: "{{ l | community.general.replace_keys(target=t) }}"

  # 2) Replace keys that starts with any of the attributes before.
- t:
    - {before: k0, after: a0}
    - {before: k1, after: a1}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='starts_with') }}"

  # 3) Replace keys that ends with any of the attributes before.
- t:
    - {before: x0, after: a0}
    - {before: x1, after: a1}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='ends_with') }}"

  # 4) Replace keys that match any regex of the attributes before.
- t:
    - {before: "^.*0_x.*$", after: a0}
    - {before: "^.*1_x.*$", after: a1}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='regex') }}"

  # The results of above examples 1-4 are all the same.
- r:
    - {a0: A0, a1: B0, k2_x2: [C0], k3_x3: foo}
    - {a0: A1, a1: B1, k2_x2: [C1], k3_x3: bar}

  # 5) If more keys match the same attribute before the last one will be used.
- t:
    - {before: "^.*_x.*$", after: X}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='regex') }}"

  # gives

- r:
    - X: foo
    - X: bar

  # 6) If there are items with equal attribute before the first one will be used.
- t:
    - {before: "^.*_x.*$", after: X}
    - {before: "^.*_x.*$", after: Y}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='regex') }}"

  # gives

- r:
    - X: foo
    - X: bar

  # 7) If there are more matches for a key the first one will be used.
- l:
    - {aaa1: A, bbb1: B, ccc1: C}
    - {aaa2: D, bbb2: E, ccc2: F}
- t:
    - {before: a, after: X}
    - {before: aa, after: Y}
  r: "{{ l | community.general.replace_keys(target=t, matching_parameter='starts_with') }}"

  # gives

- r:
    - {X: A, bbb1: B, ccc1: C}
    - {X: D, bbb2: E, ccc2: F}
"""

RETURN = r"""
_value:
  description: The list of dictionaries with replaced keys.
  type: list
  elements: dictionary
"""

from ansible_collections.community.general.plugins.plugin_utils._keys_filter import (
    _keys_filter_params,
    _keys_filter_target_dict,
)


def replace_keys(data, target=None, matching_parameter="equal"):
    """replace specific keys in a list of dictionaries"""

    # test parameters
    _keys_filter_params(data, matching_parameter)
    # test and transform target
    tz = _keys_filter_target_dict(target, matching_parameter)

    if matching_parameter == "equal":

        def replace_key(key):
            for b, a in tz:
                if key == b:
                    return a
            return key
    elif matching_parameter == "starts_with":

        def replace_key(key):
            for b, a in tz:
                if key.startswith(b):
                    return a
            return key
    elif matching_parameter == "ends_with":

        def replace_key(key):
            for b, a in tz:
                if key.endswith(b):
                    return a
            return key
    elif matching_parameter == "regex":

        def replace_key(key):
            for b, a in tz:
                if b.match(key):
                    return a
            return key

    return [{replace_key(k): v for k, v in d.items()} for d in data]


class FilterModule:
    def filters(self):
        return {
            "replace_keys": replace_keys,
        }



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_eventgrid_topic
version_added: "3.13.0"
short_description: Manage Eventgrid Topics
description:
    - Create, update ,delete and replace Eventgrid Topics.
options:
    name:
        description:
            - Name of the topic.
        required: true
        type: str
    resource_group:
        description:
            - Name of a resource group where the topic exists or will be created.
        required: true
        type: str
    location:
        description:
            - Valid azure location. Defaults to location of the resource group.
        type: str
    input_schema:
        description:
            - Schema in which incoming events will be published to this topic/domain.
              If you specify CustomEventSchema as the value for this parameter, you must
              also provide values for at least one of input_mapping_default_values /
              input_mapping_fields.
        type: str
        choices:
            - CloudEventSchemaV1_0
            - CustomEventSchema
            - EventGridSchema
        default: EventGridSchema
    input_mapping_default_values:
        description:
            - When input_schema is specified as CustomEventSchema, this parameter can be
              used to specify input mappings based on default values. You can use this
              parameter when your custom schema does not include a field that corresponds
              to one of the three fields supported by this parameter.
        type: list
        elements: dict
        suboptions:
            mapping_key:
                description:
                    - Parameter name
                type: str
                choices:
                    - subject
                    - eventtype
                    - dataversion
            mapping_value:
                description:
                    - Default value to use for Parameter when not in publish event.
                type: str
    input_mapping_fields:
        description:
            - When input_schema is specified as CustomEventSchema, this parameter is used
              to specify input mappings based on field names.
        type: list
        elements: dict
        suboptions:
            mapping_key:
                description:
                    - Parameter name
                type: str
                choices:
                    - id
                    - topic
                    - eventtime
                    - subject
                    - eventtype
                    - dataversion
            mapping_value:
                description:
                    - Corresponding value should specify the name of the field in the custom
                      input schema.  If a mapping for either 'id' or 'eventtime' is not
                      provided, Event Grid will auto-generate a default value for these two
                      fields.
                type: str
    inbound_ip_rules:
        description:
            - List of inbound IP rules
        required: false
        type: list
        default: []
        elements: dict
        suboptions:
            ipmask:
                description:
                    - IP Address in CIDR notation e.g., 10.0.0.0/8
                type: str
            action:
                description:
                    - Action to perform based ont the match or no match of the IpMask.
                type: str
                choices:
                    - Allow
                    - Deny
    public_network_access:
        description:
            - This determines if traffic is allowed over public network.
        choices:
            - enabled
            - disabled
        type: str
        default: enabled
    identity:
        description:
            - Identity for the Server.
        type: dict
        suboptions:
            type:
                description:
                    - Type of the managed identity
                choices:
                    - UserAssigned
                    - SystemAssigned
                    - SystemAssigned UserAssigned
                    - None
                default: SystemAssigned
                type: str
            user_assigned_identities:
                description:
                    - User Assigned Managed Identity
                type: list
                elements: str
    kind:
        description:
            - The kind of topic resource.
        default: Azure
        type: str
        choices:
            - Azure
            - AzureArc
    extended_location_name:
        description:
            - The extended location name.
            - Required if I(kind=AzureArc).
        type: str
    extended_location_type:
        description:
            - The extended location type.
            - Required if I(kind=AzureArc).
        type: str
        choices:
            - customlocation
    state:
        description:
            - State of the Eventgrid Topic. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    regenerate_keys:
        description:
            - if C(present) and topic exsists (ie update action) this will force
              the listed key to be regeneratd. This cannot be idempotent. Everytime
              this is called it will regenerate the key(s).
        type: list
        elements: str
        choices:
            - key1
            - key2

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create a new Topic
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    location: westus2

- name: Create a new Topic with custom input mappings
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    location: westus2
    input_schema: CustomEventSchema
    input_mapping_fields:
      - mapping_key: topic
        mapping_value: myTopicField
      - mapping_key: eventtype
        mapping_value: myEventTypeField
    input_mapping_default_values:
      - mapping_key: subject
        mapping_value: DefaultSubject
      - mapping_key: dataversion
        mapping_value: 1.0

- name: Create a new Topic that accepts events published in CloudEvents V1.0 schema
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    location: westus2
    input_schema: CloudEventSchemaV1_0

- name: Create a new Topic which allows specific inbound ip rules with Basic Sku and system assigned
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    location: westus2
    public_network_access: enabled
    inbound_ip_rules:
      - ipmask: 10.0.0.0/8
        action: Allow
      - ipmask: 10.2.0.0/8
        action: Allow
    identity:
      type: SystemAssigned

- name: Create a new topic in AzureArc targeting a custom location
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    location: eastus2euap
    kind: azurearc
    extended_location_name: /subscriptions/<subid>/resourcegroups/<rgname>/providers/microsoft.extendedlocation/customlocations/<cust-loc-name>
    extended_location_type: customlocation
    input_schema: CloudEventSchemaV1_0

- name: Delete the topic
  azure.azcollection.azure_rm_eventgrid_topic:
    resource_group: rg1
    name: topic1
    state: absent
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: true
eventgrid_topic:
    description:
        - Eventgrid Topic.
    returned: always
    type: complex
    contains:
        data_residency_boundary:
            description:
                - Residncy boundary
            returned: always
            type: str
            sample: WithinGeopair
        disable_local_auth:
            description:
                - Whether local auth is disabled
            returned: always
            type: bool
            sample: false
        endpoint:
            description:
                - Rest API endpoint to use to post events
            returned: always
            type: str
            sample: https://myTopic-20070.westus3-1.eventgrid.azure.net/api/events
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.EventGrid/topics/myTopic
        inbound_ip_rules:
            description:
                - Rules for network access
            type: complex
            returned: when-enabled
            contains:
                action:
                    description:
                        - What action will be taken, Allow or Disallow
                    returned: always
                    type: str
                    sample: Allow
                ip_mask:
                    description:
                        - CIDR notation of the network to act on.
                    returned: always
                    type: str
                    sample: 10.0.0.0/8
        input_schema:
            description:
                - The input schema assigned to this Topic
            returned: always
            type: str
            sample: EventGridSchema
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: westus3
        metric_resource_id:
            description:
                - Metric Resource ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        minimum_tls_version_allowed:
            description:
                - The minimum TLS version allowed
            returned: always
            type: str
            sample: 1.0
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: myTopic
        provisioning_state:
            description:
                - The provisioning state.
            returned: always
            type: str
            sample: Succeeded
        public_network_access:
            description:
                - Indicates if boot diagnostics are enabled.
            returned: always
            type: str
            sample: Enabled
        type:
            description:
                - Type of Eventgrid
            type: str
            returned: always
            sample: Microsoft.EventGrid/topics
        input_schema_mapping:
            description:
                - Custom input schema mapping
            type: str
            returned: when-enabled
            sample: |
                "data_version": {
                  "default_value": "1.0"
                },
                "event_time": {},
                "event_type": {
                  "source_field": "myEventTypeField"
                },
                "id": {},
                "input_schema_mapping_type": "Json",
                "subject": {
                  "default_value": "DefaultSubject"
                },
                "topic": {
                  "source_field": "myTopicField"
                }
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    from azure.mgmt.eventgrid import EventGridManagementClient
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

managed_identity_spec = dict(
    type=dict(
        type='str',
        default='SystemAssigned',
        choices=['SystemAssigned',
                 'UserAssigned',
                 'SystemAssigned UserAssigned',
                 'None'],
    ),
    user_assigned_identities=dict(
        type='list',
        elements='str'
    ),
)

input_mapping_default_values_spec = dict(
    mapping_key=dict(
        type='str',
        choices=['subject',
                 'eventtype',
                 'dataversion'],
    ),
    mapping_value=dict(
        type='str'
    ),
)

input_mapping_fields_spec = dict(
    mapping_key=dict(
        type='str',
        choices=['id',
                 'topic',
                 'eventtime',
                 'subject',
                 'eventtype',
                 'dataversion'],
    ),
    mapping_value=dict(
        type='str'
    ),
)

inbound_ip_rules_spec = dict(
    ipmask=dict(
        type='str'
    ),
    action=dict(
        type='str',
        choices=['Allow',
                 'Deny']
    )
)


class AzureRMEventgridTopic(AzureRMModuleBaseExt):

    @property
    def client(self):
        self.log('Getting client')
        if not self._client:
            self._client = self.get_mgmt_svc_client(EventGridManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)
        return self._client

    @property
    def models(self):
        self.log('Getting models')
        return self.client.topics.models

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            input_schema=dict(
                type='str',
                default='EventGridSchema',
                choices=['CloudEventSchemaV1_0', 'CustomEventSchema', 'EventGridSchema']
            ),
            input_mapping_default_values=dict(
                type='list',
                elements='dict',
                options=input_mapping_default_values_spec,
                required_together=[('mapping_key', 'mapping_value')],
            ),
            input_mapping_fields=dict(
                type='list',
                elements='dict',
                options=input_mapping_fields_spec,
                required_together=[('mapping_key', 'mapping_value')],
            ),
            inbound_ip_rules=dict(
                type='list',
                default=[],
                elements='dict',
                options=inbound_ip_rules_spec,
                required_together=[('ipmask', 'action')],
            ),
            public_network_access=dict(
                type='str',
                default='enabled',
                choices=['enabled', 'disabled']
            ),
            identity=dict(
                type='dict',
                options=managed_identity_spec
            ),
            kind=dict(
                type='str',
                default='Azure',
                choices=['Azure', 'AzureArc']
            ),
            extended_location_name=dict(
                type='str'
            ),
            extended_location_type=dict(
                type='str',
                choices=['customlocation']
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            regenerate_keys=dict(
                type='list',
                elements='str',
                choices=['key1', 'key2']
            ),
        )

        self._client = None

        self.name = None
        self.location = None
        self.input_schema = None
        self.input_mapping_default_values = None
        self.input_mapping_fields = None
        self.inbound_ip_rules = None
        self.public_network_access = None
        self.identity = None
        self.kind = None
        self.extended_location_name = None
        self.extended_location_type = None
        self.state = None
        self.regenerate_keys = None

        self.results = dict(
            changed=False,
            eventgrid_topic=dict()
        )

        required_together = [['extended_location_name', 'extended_location_type']]
        required_if = [
            ("kind", "AzureArc", ["extended_location_name"]),
        ]

        super(AzureRMEventgridTopic, self).__init__(self.module_arg_spec,
                                                    supports_tags=True,
                                                    supports_check_mode=True,
                                                    required_together=required_together,
                                                    required_if=required_if,
                                                    )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        identity_info = self._get_identity_info_only_if_not_none(self.identity, self.kind)
        input_schema_mapping = self._get_input_schema_and_mapping(self.input_mapping_fields,
                                                                  self.input_mapping_default_values)
        inbound_ip_rules = self._get_inbound_ip_rules(self.inbound_ip_rules)
        extended_location = self._get_extended_location(self.kind,
                                                        self.extended_location_name,
                                                        self.extended_location_type)

        if self.state == 'present':
            response = self.get_eventgrid_topic(self.resource_group,
                                                self.name)
            if response:
                topic_update_parameters = self.models.TopicUpdateParameters(tags=self.tags,
                                                                            public_network_access=self.public_network_access,
                                                                            inbound_ip_rules=inbound_ip_rules,
                                                                            identity=identity_info)

                self.results['compare'] = []
                changed = not self.default_compare({},
                                                   self.format_eventgrid_topic(topic_update_parameters),
                                                   response,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    topic_info = self.models.Topic(
                        location=response.get('location', self.location),
                        tags=self.tags,
                        input_schema=response.get('input_schema', self.input_schema),
                        input_schema_mapping=input_schema_mapping,
                        public_network_access=self.public_network_access,
                        inbound_ip_rules=inbound_ip_rules,
                        identity=identity_info,
                        disable_local_auth=response.get('disable_local_auth', False),
                        data_residency_boundary=response.get('data_residency_boundary'),
                        minimum_tls_version_allowed=response.get('minimum_tls_version_allowed'),
                        kind=self.kind,
                        extended_location=extended_location)
                    response = self.begin_create_or_update_eventgrid_topic(self.resource_group,
                                                                           self.name,
                                                                           topic_info)
                if self.regenerate_keys and not self.check_mode:
                    changed = True
                    for key_name in self.regenerate_keys:
                        key_response = self.topic_regenerate_key(self.resource_group,
                                                                 self.name,
                                                                 key_name)

            else:
                changed = True
                topic_info = self.models.Topic(
                    location=self.location,
                    tags=self.tags,
                    input_schema=self.input_schema,
                    input_schema_mapping=input_schema_mapping,
                    public_network_access=self.public_network_access,
                    inbound_ip_rules=inbound_ip_rules,
                    identity=identity_info,
                    kind=self.kind,
                    extended_location=extended_location)

                if not self.check_mode:
                    response = self.begin_create_or_update_eventgrid_topic(self.resource_group,
                                                                           self.name,
                                                                           topic_info)
        else:
            response = self.get_eventgrid_topic(self.resource_group,
                                                self.name)
            if response:
                changed = True
                if not self.check_mode:
                    response = self.delete_eventgrid_topic(self.resource_group,
                                                           self.name)

        self.results['changed'] = changed
        self.results['eventgrid_topic'] = response

        return self.results

    def begin_create_or_update_eventgrid_topic(self, resource_group, topic_name, topic_info):
        self.log('Creates or updates the Eventgrid Topic.')

        try:
            response = self.client.topics.begin_create_or_update(resource_group, topic_name, topic_info)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail('Creates or updates the Eventgrid Topic got Exception as as {0}'.format(exc.message or str(exc)))
        return self.format_eventgrid_topic(response)

    def begin_update_eventgrid_topic(self, resource_group, topic_name, topic_update_parameters):
        self.log('Selectively updates the Eventgrid Topic.')

        try:
            response = self.client.topics.begin_update(resource_group_name=resource_group,
                                                       topic_name=topic_name,
                                                       topic_update_parameters=topic_update_parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail('Selectively updates the Eventgrid Topic got Excption as {0}'.format(exc.message or str(exc)))

        return self.format_eventgrid_topic(response)

    def delete_eventgrid_topic(self, resource_group, topic_name):
        self.log('Deletes the entire Eventgrid Topic.')
        try:
            self.client.topics.begin_delete(resource_group, topic_name)
        except Exception as exc:
            self.fail('Delete the entire set of tag got Excetion as {0}'.format(exc.message or str(exc)))

    def get_eventgrid_topic(self, resource_group, topic_name):
        self.log('Get properties for {0}'.format(topic_name))
        try:
            response = self.client.topics.get(resource_group, topic_name)
            if response is not None:
                return self.format_eventgrid_topic(response)
        except ResourceNotFoundError:
            self.log('Did not find the eventgrid_topic.')
            return False
        except Exception as exc:
            self.fail('Error when get eventgrid_topic got Excetion as {0}'.format(exc.message or str(exc)))

    def format_eventgrid_topic(self, topic_data):
        results = topic_data and topic_data.as_dict() or {}
        return results

    def _get_identity_info_only_if_not_none(self, identity=None, kind=None):
        identity_info = None
        if identity:
            # If UserAssigned in module args and ids specified
            if 'UserAssigned' in identity.get('type') and len(identity.get('user_assigned_identities', [])) != 0:
                user_assigned_identities_dict = {uami: self.models.UserIdentityProperties() for uami in identity.get('user_assigned_identities')}
                # Append identities to the model
                identity_info = self.models.IdentityInfo(
                    type=identity.get('type'),
                    user_assigned_identities=user_assigned_identities_dict
                )
            # If UserAssigned in module args, but ids are not specified
            elif 'UserAssigned' in identity.get('type') and len(identity.get('user_assigned_identities', [])) == 0:
                self.fail("UserAssigned specified but no User Identity IDs provided")
            # In any other case ('SystemAssigned' or 'None') apply the configuration to the model
            else:
                identity_info = self.models.IdentityInfo(
                    type=identity.get('type')
                )
        else:
            if kind is None or kind == 'Azure':
                indentity_info = self.models.IdentityInfo(type="None")

        return identity_info

    def _get_input_schema_and_mapping(self,
                                      input_mapping_fields=None,
                                      input_mapping_default_values=None):

        input_schema_mapping = None

        if input_mapping_fields is not None or input_mapping_default_values is not None:
            input_schema_mapping = self.models.JsonInputSchemaMapping()

            input_schema_mapping.id = self.models.JsonField()
            input_schema_mapping.topic = self.models.JsonField()
            input_schema_mapping.event_time = self.models.JsonField()
            input_schema_mapping.subject = self.models.JsonFieldWithDefault()
            input_schema_mapping.event_type = self.models.JsonFieldWithDefault()
            input_schema_mapping.data_version = self.models.JsonFieldWithDefault()

            if input_mapping_fields is not None:
                for input_mapping_field in input_mapping_fields:
                    target = input_mapping_field.get('mapping_key')
                    source = input_mapping_field.get('mapping_value')
                    if target == 'id':
                        input_schema_mapping.id.source_field = source
                    elif target == 'eventtime':
                        input_schema_mapping.event_time.source_field = source
                    elif target == 'topic':
                        input_schema_mapping.topic.source_field = source
                    elif target == 'subject':
                        input_schema_mapping.subject.source_field = source
                    elif target == 'dataversion':
                        input_schema_mapping.data_version.source_field = source
                    elif target == 'eventtype':
                        input_schema_mapping.event_type.source_field = source

            if input_mapping_default_values is not None:
                for input_mapping_default_value in input_mapping_default_values:
                    target = input_mapping_default_value.get('mapping_key')
                    source = input_mapping_default_value.get('mapping_value')
                    if target == 'subject':
                        input_schema_mapping.subject.default_value = source
                    elif target == 'dataversion':
                        input_schema_mapping.data_version.default_value = source
                    elif target == 'eventtype':
                        input_schema_mapping.event_type.default_value = source

        return input_schema_mapping

    def _get_inbound_ip_rules(self, inbound_ip_rules):
        inbound_ip_rules_mapping = []

        for inbound_ip_rule in inbound_ip_rules:
            ipmask = inbound_ip_rule.get('ipmask')
            action = inbound_ip_rule.get('action')
            inbound_ip_rules_mapping.append(self.models.InboundIpRule(ip_mask=ipmask, action=action))

        return inbound_ip_rules_mapping

    def _get_extended_location(self, kind_name=None, extended_location_name=None,
                               extended_location_type=None):
        result = None

        if kind_name == 'AzureArc':
            result = self.models.ExtendedLocation(name=extended_location_name,
                                                  type=extended_location_type)

        return result

    def topic_regenerate_key(self,
                             resource_group,
                             topic_name,
                             key_name):
        regenerate_key_request = self.models.TopicRegenerateKeyRequest(key_name=key_name)

        return self.client.topics.begin_regenerate_key(
            resource_group_name=resource_group,
            topic_name=topic_name,
            regenerate_key_request=regenerate_key_request
        )


def main():
    AzureRMEventgridTopic()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_sm_devices_install_apps
short_description: Resource module for networks _sm _devices _install _apps
description:
  - Manage operation create of the resource networks _sm _devices _install _apps.
  - Install applications on a device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  appIds:
    description: Ids of applications to be installed.
    elements: str
    type: list
  deviceId:
    description: DeviceId path parameter. Device ID.
    type: str
  force:
    description: By default, installation of an app which is believed to already be
      present on the device will be skipped. If you'd like to force the installation
      of the app, set this parameter to true.
    type: bool
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sm installNetworkSmDeviceApps
    description: Complete reference of the installNetworkSmDeviceApps API.
    link: https://developer.cisco.com/meraki/api-v1/#!install-network-sm-device-apps
notes:
  - SDK Method used are
    sm.Sm.install_network_sm_device_apps,
  - Paths used are
    post /networks/{networkId}/sm/devices/{deviceId}/installApps,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_sm_devices_install_apps:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    appIds:
      - '1284392014819'
      - '2983092129865'
    deviceId: string
    force: false
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard documentation fragment
    DOCUMENTATION = r"""
options:
  auth_keycloak_url:
    description:
      - URL to the Keycloak instance.
    type: str
    required: true
    aliases:
      - url

  auth_client_id:
    description:
      - OpenID Connect C(client_id) to authenticate to the API with.
    type: str
    default: admin-cli

  auth_realm:
    description:
      - Keycloak realm name to authenticate to for API access.
    type: str

  auth_client_secret:
    description:
      - Client Secret to use in conjunction with O(auth_client_id) (if required).
    type: str

  auth_username:
    description:
      - Username to authenticate for API access with.
    type: str
    aliases:
      - username

  auth_password:
    description:
      - Password to authenticate for API access with.
    type: str
    aliases:
      - password

  token:
    description:
      - Authentication token for Keycloak API.
    type: str
    version_added: 3.0.0

  refresh_token:
    description:
      - Authentication refresh token for Keycloak API.
    type: str
    version_added: 10.3.0

  validate_certs:
    description:
      - Verify TLS certificates (do not disable this in production).
    type: bool
    default: true

  connection_timeout:
    description:
      - Controls the HTTP connections timeout period (in seconds) to Keycloak API.
    type: int
    default: 10
    version_added: 4.5.0

  http_agent:
    description:
      - Configures the HTTP User-Agent header.
    type: str
    default: Ansible
    version_added: 5.4.0
"""

    ACTIONGROUP_KEYCLOAK = r"""
options: {}
attributes:
  action_group:
    description: Use C(group/community.general.keycloak) in C(module_defaults) to set defaults for this module.
    support: full
    membership:
      - community.general.keycloak
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_simple_cluster
short_description: Manages simple-cluster objects on Checkpoint over Web Services API
description:
  - Manages simple-cluster objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80.40 management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  ip_address:
    description:
      - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
    type: str
  ipv4_address:
    description:
      - IPv4 address.
    type: str
  ipv6_address:
    description:
      - IPv6 address.
    type: str
  anti_bot:
    description:
      - Anti-Bot blade enabled.
    type: bool
  anti_virus:
    description:
      - Anti-Virus blade enabled.
    type: bool
  application_control:
    description:
      - Application Control blade enabled.
    type: bool
  cluster_mode:
    description:
      - Cluster mode.
    type: str
    choices: ['cluster-xl-ha', 'cluster-ls-multicast', 'cluster-ls-unicast', 'opsec-ha', 'opsec-ls']
  content_awareness:
    description:
      - Content Awareness blade enabled.
    type: bool
  enable_https_inspection:
    description:
      - Enable HTTPS Inspection after defining an outbound inspection certificate. <br>To define the outbound certificate use outbound inspection
        certificate API.
    type: bool
  firewall:
    description:
      - Firewall blade enabled.
    type: bool
  firewall_settings:
    description:
      - N/A
    type: dict
    suboptions:
      auto_calculate_connections_hash_table_size_and_memory_pool:
        description:
          - N/A
        type: bool
      auto_maximum_limit_for_concurrent_connections:
        description:
          - N/A
        type: bool
      connections_hash_size:
        description:
          - N/A
        type: int
      maximum_limit_for_concurrent_connections:
        description:
          - N/A
        type: int
      maximum_memory_pool_size:
        description:
          - N/A
        type: int
      memory_pool_size:
        description:
          - N/A
        type: int
  hardware:
    description:
      - Cluster platform hardware.
    type: str
  interfaces:
    description:
      - N/A
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Object name.
        type: str
      interface_type:
        description:
          - Cluster interface type.
        type: str
        choices: ['cluster', 'sync', 'cluster + sync', 'private']
      ip_address:
        description:
          - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
        type: str
      ipv4_address:
        description:
          - IPv4 address.
        type: str
      ipv6_address:
        description:
          - IPv6 address.
        type: str
      network_mask:
        description:
          - IPv4 or IPv6 network mask. If both masks are required use ipv4-network-mask and ipv6-network-mask fields explicitly. Instead of
            providing mask itself it is possible to specify IPv4 or IPv6 mask length in mask-length field. If both masks length are required use
            ipv4-mask-length and  ipv6-mask-length fields explicitly.
        type: str
      ipv4_network_mask:
        description:
          - IPv4 network address.
        type: str
      ipv6_network_mask:
        description:
          - IPv6 network address.
        type: str
      mask_length:
        description:
          - IPv4 or IPv6 network mask length.
        type: str
      ipv4_mask_length:
        description:
          - IPv4 network mask length.
        type: str
      ipv6_mask_length:
        description:
          - IPv6 network mask length.
        type: str
      anti_spoofing:
        description:
          - N/A
        type: bool
      anti_spoofing_settings:
        description:
          - N/A
        type: dict
        suboptions:
          action:
            description:
              - If packets will be rejected (the Prevent option) or whether the packets will be monitored (the Detect option).
            type: str
            choices: ['prevent', 'detect']
          exclude_packets:
            description:
              - Don't check packets from excluded network.
              - Available from R81.10 management version.
            type: bool
          excluded_network_name:
            description:
              - Excluded network name.
              - Available from R81.10 management version.
            type: str
          excluded_network_uid:
            description:
              - Excluded network UID.
              - Available from R81.10 management version.
            type: str
          spoof_tracking:
            description:
              - Spoof tracking.
              - Available from R81.10 management version.
            type: str
            choices: ['none', 'log', 'alert']
      multicast_address:
        description:
          - Multicast IP Address.
        type: str
      multicast_address_type:
        description:
          - Multicast Address Type.
        type: str
        choices: ['manual', 'default']
      security_zone:
        description:
          - N/A
        type: bool
      security_zone_settings:
        description:
          - N/A
        type: dict
        suboptions:
          auto_calculated:
            description:
              - Security Zone is calculated according to where the interface leads to.
            type: bool
          specific_zone:
            description:
              - Security Zone specified manually.
            type: str
      tags:
        description:
          - Collection of tag identifiers.
        type: list
        elements: str
      topology:
        description:
          - N/A
        type: str
        choices: ['automatic', 'external', 'internal']
      topology_settings:
        description:
          - N/A
        type: dict
        suboptions:
          interface_leads_to_dmz:
            description:
              - Whether this interface leads to demilitarized zone (perimeter network).
            type: bool
          ip_address_behind_this_interface:
            description:
              - Network settings behind this interface.
            type: str
            choices: ['not defined', 'network defined by the interface ip and net mask', 'network defined by routing', 'specific']
          specific_network:
            description:
              - Network behind this interface.
            type: str
      color:
        description:
          - Color of the object. Should be one of existing colors.
        type: str
        choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange',
                 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray',
                 'light green', 'lemon chiffon', 'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive',
                 'orange', 'red', 'sienna', 'yellow']
      comments:
        description:
          - Comments string.
        type: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
  ips:
    description:
      - Intrusion Prevention System blade enabled.
    type: bool
  ips_settings:
    description:
      - Cluster IPS settings.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      bypass_all_under_load:
        description:
          - Disable/enable all IPS protections until CPU and memory levels are back to normal.
        type: bool
      bypass_track_method:
        description:
          - Track options when all IPS protections are disabled until CPU/memory levels are back to normal.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
      activation_mode:
        description:
          - Defines whether the IPS blade operates in Detect Only mode or enforces the configured IPS Policy.
        type: str
        choices: ['according-to-policy', 'detect-only']
      cpu_usage_low_threshold:
        description:
          - CPU usage low threshold percentage (1-99).
        type: int
      cpu_usage_high_threshold:
        description:
          - CPU usage high threshold percentage (1-99).
        type: int
      memory_usage_low_threshold:
        description:
          - Memory usage low threshold percentage (1-99).
        type: int
      memory_usage_high_threshold:
        description:
          - Memory usage high threshold percentage (1-99).
        type: int
      send_threat_cloud_info:
        description:
          - Help improve Check Point Threat Prevention product by sending anonymous information.
        type: bool
      reject_on_cluster_fail_over:
        description:
          - Define the IPS connections during fail over reject packets or accept packets.
        type: bool
  members:
    description:
      - Cluster members list. Only new cluster member can be added. Adding existing gateway is not supported.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Object name.
        type: str
      interfaces:
        description:
          - Cluster Member network interfaces.
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - Object name.
            type: str
          anti_spoofing:
            description:
              - N/A
            type: bool
          anti_spoofing_settings:
            description:
              - N/A
            type: dict
            suboptions:
              action:
                description:
                  - If packets will be rejected (the Prevent option) or whether the packets will be monitored (the Detect option).
                type: str
                choices: ['prevent', 'detect']
              exclude_packets:
                description:
                  - Don't check packets from excluded network.
                  - Available from R81.10 management version.
                type: bool
              excluded_network_name:
                description:
                  - Excluded network name.
                  - Available from R81.10 management version.
                type: str
              excluded_network_uid:
                description:
                  - Excluded network UID.
                  - Available from R81.10 management version.
                type: str
              spoof_tracking:
                description:
                  - Spoof tracking.
                  - Available from R81.10 management version.
                type: str
                choices: ['none', 'log', 'alert']
          ip_address:
            description:
              - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
            type: str
          ipv4_address:
            description:
              - IPv4 address.
            type: str
          ipv6_address:
            description:
              - IPv6 address.
            type: str
          network_mask:
            description:
              - IPv4 or IPv6 network mask. If both masks are required use ipv4-network-mask and ipv6-network-mask fields explicitly. Instead
                of providing mask itself it is possible to specify IPv4 or IPv6 mask length in mask-length field. If both masks length are required use
                ipv4-mask-length and  ipv6-mask-length fields explicitly.
            type: str
          ipv4_network_mask:
            description:
              - IPv4 network address.
            type: str
          ipv6_network_mask:
            description:
              - IPv6 network address.
            type: str
          mask_length:
            description:
              - IPv4 or IPv6 network mask length.
            type: str
          ipv4_mask_length:
            description:
              - IPv4 network mask length.
            type: str
          ipv6_mask_length:
            description:
              - IPv6 network mask length.
            type: str
          security_zone:
            description:
              - N/A
            type: bool
          security_zone_settings:
            description:
              - N/A
            type: dict
            suboptions:
              auto_calculated:
                description:
                  - Security Zone is calculated according to where the interface leads to.
                type: bool
              specific_zone:
                description:
                  - Security Zone specified manually.
                type: str
          tags:
            description:
              - Collection of tag identifiers.
            type: list
            elements: str
          topology:
            description:
              - N/A
            type: str
            choices: ['automatic', 'external', 'internal']
          topology_settings:
            description:
              - N/A
            type: dict
            suboptions:
              interface_leads_to_dmz:
                description:
                  - Whether this interface leads to demilitarized zone (perimeter network).
                type: bool
              ip_address_behind_this_interface:
                description:
                  - Network settings behind this interface.
                type: str
                choices: ['not defined', 'network defined by the interface ip and net mask', 'network defined by routing', 'specific']
              specific_network:
                description:
                  - Network behind this interface.
                type: str
          color:
            description:
              - Color of the object. Should be one of existing colors.
            type: str
            choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange',
                     'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray',
                     'light green', 'lemon chiffon', 'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive',
                     'orange', 'red', 'sienna', 'yellow']
          comments:
            description:
              - Comments string.
            type: str
          details_level:
            description:
              - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully
                detailed representation of the object.
            type: str
            choices: ['uid', 'standard', 'full']
          ignore_warnings:
            description:
              - Apply changes ignoring warnings.
            type: bool
          ignore_errors:
            description:
              - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings
                will also be ignored.
            type: bool
      ip_address:
        description:
          - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
        type: str
      ipv4_address:
        description:
          - IPv4 address.
        type: str
      ipv6_address:
        description:
          - IPv6 address.
        type: str
      one_time_password:
        description:
          - N/A
        type: str
      tags:
        description:
          - Collection of tag identifiers.
        type: list
        elements: str
      color:
        description:
          - Color of the object. Should be one of existing colors.
        type: str
        choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange',
                 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray',
                 'light green', 'lemon chiffon', 'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive',
                 'orange', 'red', 'sienna', 'yellow']
      comments:
        description:
          - Comments string.
        type: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
  os_name:
    description:
      - Cluster platform operating system.
    type: str
  platform_portal_settings:
    description:
      - Platform portal settings.
      - Available from R81 JHF management version.
    type: dict
    suboptions:
      portal_web_settings:
        description:
          - Configuration of the portal web settings.
        type: dict
        suboptions:
          aliases:
            description:
              - List of URL aliases that are redirected to the main portal URL.
            type: list
            elements: str
          ip_address:
            description:
              - Optional, IP address for the web portal to use, if your DNS server fails to resolve the main portal URL.
                Note, If your DNS server resolves the main portal URL, this IP address is ignored.
            type: str
          main_url:
            description:
              - The main URL for the web portal.
            type: str
      certificate_settings:
        description:
          - Configuration of the portal certificate settings.
        type: dict
        suboptions:
          base64_certificate:
            description:
              - The certificate file encoded in Base64 with padding. This file must be in the *.p12 format.
            type: str
          base64_password:
            description:
              - Password (encoded in Base64 with padding) for the certificate file.
            type: str
      accessibility:
        description:
          - Configuration of the portal access settings.
        type: dict
        suboptions:
          allow_access_from:
            description:
              - Allowed access to the web portal (based on interfaces, or security policy).
            type: str
            choices: ['rule_base', 'internal_interfaces', 'all_interfaces']
          internal_access_settings:
            description:
              - Configuration of the additional portal access settings for internal interfaces only.
            type: dict
            suboptions:
              undefined:
                description:
                  - Controls portal access settings for internal interfaces, whose topology is set to 'Undefined'.
                type: bool
              dmz:
                description:
                  - Controls portal access settings for internal interfaces, whose topology is set to 'DMZ'.
                type: bool
              vpn:
                description:
                  - Controls portal access settings for interfaces that are part of a VPN Encryption Domain.
                type: bool
  send_alerts_to_server:
    description:
      - Server(s) to send alerts to.
    type: list
    elements: str
  send_logs_to_backup_server:
    description:
      - Backup server(s) to send logs to.
    type: list
    elements: str
  send_logs_to_server:
    description:
      - Server(s) to send logs to.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  threat_emulation:
    description:
      - Threat Emulation blade enabled.
    type: bool
  threat_extraction:
    description:
      - Threat Extraction blade enabled.
      - Available from R81.10 management version.
    type: bool
  threat_prevention_mode:
    description:
      - The mode of Threat Prevention to use. When using Autonomous Threat Prevention, disabling the Threat Prevention blades is not allowed.
      - Available from R81.10 management version.
    type: str
    choices: ['autonomous', 'custom']
  url_filtering:
    description:
      - URL Filtering blade enabled.
    type: bool
  usercheck_portal_settings:
    description:
      - UserCheck portal settings.
      - Available from R81 JHF management version.
    type: dict
    suboptions:
      enabled:
        description:
          - State of the web portal (enabled or disabled). The supported blades are, {'Application Control', 'URL Filtering', 'Data Loss
            Prevention', 'Anti Virus', 'Anti Bot', 'Threat Emulation', 'Threat Extraction', 'Data Awareness'}.
        type: bool
      portal_web_settings:
        description:
          - Configuration of the portal web settings.
        type: dict
        suboptions:
          aliases:
            description:
              - List of URL aliases that are redirected to the main portal URL.
            type: list
            elements: str
          ip_address:
            description:
              - Optional, IP address for the web portal to use, if your DNS server fails to resolve the main portal URL.
                Note, If your DNS server resolves the main portal URL, this IP address is ignored.
            type: str
          main_url:
            description:
              - The main URL for the web portal.
            type: str
      certificate_settings:
        description:
          - Configuration of the portal certificate settings.
        type: dict
        suboptions:
          base64_certificate:
            description:
              - The certificate file encoded in Base64 with padding. This file must be in the *.p12 format.
            type: str
          base64_password:
            description:
              - Password (encoded in Base64 with padding) for the certificate file.
            type: str
      accessibility:
        description:
          - Configuration of the portal access settings.
        type: dict
        suboptions:
          allow_access_from:
            description:
              - Allowed access to the web portal (based on interfaces, or security policy).
            type: str
            choices: ['rule_base', 'internal_interfaces', 'all_interfaces']
          internal_access_settings:
            description:
              - Configuration of the additional portal access settings for internal interfaces only.
            type: dict
            suboptions:
              undefined:
                description:
                  - Controls portal access settings for internal interfaces, whose topology is set to 'Undefined'.
                type: bool
              dmz:
                description:
                  - Controls portal access settings for internal interfaces, whose topology is set to 'DMZ'.
                type: bool
              vpn:
                description:
                  - Controls portal access settings for interfaces that are part of a VPN Encryption Domain.
                type: bool
  cluster_version:
    description:
      - Cluster platform version.
    type: str
  vpn:
    description:
      - VPN blade enabled.
    type: bool
  vpn_settings:
    description:
      - Gateway VPN settings.
    type: dict
    suboptions:
      authentication:
        description:
          - Authentication.
          - Available from R81 management version.
        type: dict
        suboptions:
          authentication_clients:
            description:
              - Collection of VPN Authentication clients identified by the name or UID.
            type: list
            elements: str
      link_selection:
        description:
          - Link Selection.
          - Available from R81 management version.
        type: dict
        suboptions:
          ip_selection:
            description:
              - N/A
            type: str
            choices: ['use-main-address', 'use-selected-address-from-topology', 'use-statically-nated-ip',
                     'calculated-ip-based-on-topology', 'dns-resolving-from-hostname', 'dns-resolving-from-gateway-and-domain-name',
                     'use-probing-with-high-availability', 'use-probing-with-load-sharing', 'use-one-time-probing']
          dns_resolving_hostname:
            description:
              - DNS Resolving Hostname. Must be set when "ip-selection" was selected to be "dns-resolving-from-hostname".
            type: str
          ip_address:
            description:
              - IP Address. Must be set when "ip-selection" was selected to be "use-selected-address-from-topology" or "use-statically-nated-ip".
            type: str
      maximum_concurrent_ike_negotiations:
        description:
          - N/A
        type: int
      maximum_concurrent_tunnels:
        description:
          - N/A
        type: int
      office_mode:
        description:
          - Office Mode. Notation Wide Impact - Office Mode apply IPSec VPN Software Blade clients and to the Mobile Access Software Blade clients.
          - Available from R81 management version.
        type: dict
        suboptions:
          mode:
            description:
              - Office Mode Permissions.When selected to be "off", all the other definitions are irrelevant.
            type: str
            choices: ['off', 'specific-group', 'all-users']
          group:
            description:
              - Group. Identified by name or UID. Must be set when "office-mode-permissions" was selected to be "group".
            type: str
          allocate_ip_address_from:
            description:
              - Allocate IP address Method.
                Allocate IP address by sequentially trying the given methods until success.
            type: dict
            suboptions:
              radius_server:
                description:
                  - Radius server used to authenticate the user.
                type: bool
              use_allocate_method:
                description:
                  - Use Allocate Method.
                type: bool
              allocate_method:
                description:
                  - Using either Manual (IP Pool) or Automatic (DHCP).
                    Must be set when "use-allocate-method" is true.
                type: str
                choices: ['manual', 'automatic']
              manual_network:
                description:
                  - Manual Network. Identified by name or UID.
                    Must be set when "allocate-method" was selected to be "manual".
                type: str
              dhcp_server:
                description:
                  - DHCP Server. Identified by name or UID.
                    Must be set when "allocate-method" was selected to be "automatic".
                type: str
              virtual_ip_address:
                description:
                  - Virtual IPV4 address for DHCP server replies.
                    Must be set when "allocate-method" was selected to be "automatic".
                type: str
              dhcp_mac_address:
                description:
                  - Calculated MAC address for DHCP allocation.
                    Must be set when "allocate-method" was selected to be "automatic".
                type: str
                choices: ['per-machine', 'per-user']
              optional_parameters:
                description:
                  - This configuration applies to all Office Mode methods except Automatic (using DHCP) and ipassignment.conf entries which contain this data.
                type: dict
                suboptions:
                  use_primary_dns_server:
                    description:
                      - Use Primary DNS Server.
                    type: bool
                  primary_dns_server:
                    description:
                      - Primary DNS Server. Identified by name or UID.
                        Must be set when "use-primary-dns-server" is true and can not be set when "use-primary-dns-server" is false.
                    type: str
                  use_first_backup_dns_server:
                    description:
                      - Use First Backup DNS Server.
                    type: bool
                  first_backup_dns_server:
                    description:
                      - First Backup DNS Server. Identified by name or UID.
                        Must be set when "use-first-backup-dns-server" is true and can not be set when "use-first-backup-dns-server" is false.
                    type: str
                  use_second_backup_dns_server:
                    description:
                      - Use Second Backup DNS Server.
                    type: bool
                  second_backup_dns_server:
                    description:
                      - Second Backup DNS Server. Identified by name or UID.
                        Must be set when "use-second-backup-dns-server" is true and can not be set when "use-second-backup-dns-server" is false.
                    type: str
                  dns_suffixes:
                    description:
                      - DNS Suffixes.
                    type: str
                  use_primary_wins_server:
                    description:
                      - Use Primary WINS Server.
                    type: bool
                  primary_wins_server:
                    description:
                      - Primary WINS Server. Identified by name or UID.
                        Must be set when "use-primary-wins-server" is true and can not be set when "use-primary-wins-server" is false.
                    type: str
                  use_first_backup_wins_server:
                    description:
                      - Use First Backup WINS Server.
                    type: bool
                  first_backup_wins_server:
                    description:
                      - First Backup WINS Server. Identified by name or UID.
                        Must be set when "use-first-backup-wins-server" is true and can not be set when "use-first-backup-wins-server" is false.
                    type: str
                  use_second_backup_wins_server:
                    description:
                      - Use Second Backup WINS Server.
                    type: bool
                  second_backup_wins_server:
                    description:
                      - Second Backup WINS Server. Identified by name or UID.
                        Must be set when "use-second-backup-wins-server" is true and can not be set when "use-second-backup-wins-server" is false.
                    type: str
                  ip_lease_duration:
                    description:
                      - IP Lease Duration in Minutes. The value must be in the range 2-32767.
                    type: int
          support_multiple_interfaces:
            description:
              - Support connectivity enhancement for gateways with multiple external interfaces.
            type: bool
          perform_anti_spoofing:
            description:
              - Perform Anti-Spoofing on Office Mode addresses.
            type: bool
          anti_spoofing_additional_addresses:
            description:
              - Additional IP Addresses for Anti-Spoofing. Identified by name or UID.
                Must be set when "perform-anti-spoofings" is true.
            type: str
      remote_access:
        description:
          - Remote Access.
          - Available from R81 management version.
        type: dict
        suboptions:
          support_l2tp:
            description:
              - Support L2TP (relevant only when office mode is active).
            type: bool
          l2tp_auth_method:
            description:
              - L2TP Authentication Method.
                Must be set when "support-l2tp" is true.
            type: str
            choices: ['certificate', 'md5']
          l2tp_certificate:
            description:
              - L2TP Certificate.
                Must be set when "l2tp-auth-method" was selected to be "certificate".
                Insert "defaultCert" when you want to use the default certificate.
            type: str
          allow_vpn_clients_to_route_traffic:
            description:
              - Allow VPN clients to route traffic.
            type: bool
          support_nat_traversal_mechanism:
            description:
              - Support NAT traversal mechanism (UDP encapsulation).
            type: bool
          nat_traversal_service:
            description:
              - Allocated NAT traversal UDP service. Identified by name or UID.
                Must be set when "support-nat-traversal-mechanism" is true.
            type: str
          support_visitor_mode:
            description:
              - Support Visitor Mode.
            type: bool
          visitor_mode_service:
            description:
              - TCP Service for Visitor Mode. Identified by name or UID.
                Must be set when "support-visitor-mode" is true.
            type: str
          visitor_mode_interface:
            description:
              - Interface for Visitor Mode.
                Must be set when "support-visitor-mode" is true.
                Insert IPV4 Address of existing interface or "All IPs" when you want all interfaces.
            type: str
      vpn_domain:
        description:
          - Gateway VPN domain identified by the name or UID.
        type: str
      vpn_domain_type:
        description:
          - Gateway VPN domain type.
        type: str
        choices: ['manual', 'addresses_behind_gw']
  show_portals_certificate:
    description:
      - Indicates whether to show the portals certificate value in the reply.
      - Available from R81 JHF management version.
    type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  groups:
    description:
      - Collection of group identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-simple-cluster
  cp_mgmt_simple_cluster:
    cluster_mode: cluster-xl-ha
    color: yellow
    firewall: true
    interfaces:
      - anti_spoofing: true
        interface_type: cluster
        ip_address: 17.23.5.1
        name: eth0
        network_mask: 255.255.255.0
        topology: EXTERNAL
      - interface_type: sync
        name: eth1
        topology: INTERNAL
        topology_settings:
          interface_leads_to_dmz: false
          ip_address_behind_this_interface: network defined by the interface ip and net mask
      - anti_spoofing: true
        interface_type: cluster
        ip_address: 192.168.1.1
        name: eth2
        network_mask: 255.255.255.0
        topology: INTERNAL
        topology_settings:
          interface_leads_to_dmz: false
          ip_address_behind_this_interface: network defined by the interface ip and net mask
    ip_address: 17.23.5.1
    members:
      - interfaces:
          - ip_address: 17.23.5.2
            name: eth0
            network_mask: 255.255.255.0
          - ip_address: 1.1.2.4
            name: eth1
            network_mask: 255.255.255.0
          - ip_address: 192.168.1.2
            name: eth2
            network_mask: 255.255.255.0
        ip_address: 17.23.5.2
        name: member1
        one_time_password: abcd
      - interfaces:
          - ip_address: 17.23.5.3
            name: eth0
            network_mask: 255.255.255.0
          - ip_address: 1.1.2.5
            name: eth1
            network_mask: 255.255.255.0
          - ip_address: 192.168.1.3
            name: eth2
            network_mask: 255.255.255.0
        ip_address: 17.23.5.3
        name: member2
        one_time_password: abcd
    name: cluster1
    os_name: Gaia
    state: present
    cluster_version: R80.30

- name: set-simple-cluster
  cp_mgmt_simple_cluster:
    name: cluster1
    state: present

- name: delete-simple-cluster
  cp_mgmt_simple_cluster:
    name: cluster1
    state: absent
"""

RETURN = """
cp_mgmt_simple_cluster:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        ip_address=dict(type="str"),
        ipv4_address=dict(type="str"),
        ipv6_address=dict(type="str"),
        anti_bot=dict(type="bool"),
        anti_virus=dict(type="bool"),
        application_control=dict(type="bool"),
        cluster_mode=dict(
            type="str",
            choices=[
                "cluster-xl-ha",
                "cluster-ls-multicast",
                "cluster-ls-unicast",
                "opsec-ha",
                "opsec-ls",
            ],
        ),
        content_awareness=dict(type="bool"),
        enable_https_inspection=dict(type='bool'),
        firewall=dict(type="bool"),
        firewall_settings=dict(
            type="dict",
            options=dict(
                auto_calculate_connections_hash_table_size_and_memory_pool=dict(
                    type="bool"
                ),
                auto_maximum_limit_for_concurrent_connections=dict(
                    type="bool"
                ),
                connections_hash_size=dict(type="int"),
                maximum_limit_for_concurrent_connections=dict(type="int"),
                maximum_memory_pool_size=dict(type="int"),
                memory_pool_size=dict(type="int"),
            ),
        ),
        hardware=dict(type="str"),
        interfaces=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str"),
                interface_type=dict(
                    type="str",
                    choices=["cluster", "sync", "cluster + sync", "private"],
                ),
                ip_address=dict(type="str"),
                ipv4_address=dict(type="str"),
                ipv6_address=dict(type="str"),
                network_mask=dict(type="str"),
                ipv4_network_mask=dict(type="str"),
                ipv6_network_mask=dict(type="str"),
                mask_length=dict(type="str"),
                ipv4_mask_length=dict(type="str"),
                ipv6_mask_length=dict(type="str"),
                anti_spoofing=dict(type="bool"),
                anti_spoofing_settings=dict(
                    type="dict",
                    options=dict(
                        action=dict(type="str", choices=["prevent", "detect"]),
                        exclude_packets=dict(type="bool"),
                        excluded_network_name=dict(type="str"),
                        excluded_network_uid=dict(type="str"),
                        spoof_tracking=dict(
                            type="str", choices=["none", "log", "alert"]
                        ),
                    ),
                ),
                multicast_address=dict(type="str"),
                multicast_address_type=dict(
                    type="str", choices=["manual", "default"]
                ),
                security_zone=dict(type="bool"),
                security_zone_settings=dict(
                    type="dict",
                    options=dict(
                        auto_calculated=dict(type="bool"),
                        specific_zone=dict(type="str"),
                    ),
                ),
                tags=dict(type="list", elements="str"),
                topology=dict(
                    type="str", choices=["automatic", "external", "internal"]
                ),
                topology_settings=dict(
                    type="dict",
                    options=dict(
                        interface_leads_to_dmz=dict(type="bool"),
                        ip_address_behind_this_interface=dict(
                            type="str",
                            choices=[
                                "not defined",
                                "network defined by the interface ip and net mask",
                                "network defined by routing",
                                "specific",
                            ],
                        ),
                        specific_network=dict(type="str"),
                    ),
                ),
                color=dict(
                    type="str",
                    choices=[
                        "aquamarine",
                        "black",
                        "blue",
                        "crete blue",
                        "burlywood",
                        "cyan",
                        "dark green",
                        "khaki",
                        "orchid",
                        "dark orange",
                        "dark sea green",
                        "pink",
                        "turquoise",
                        "dark blue",
                        "firebrick",
                        "brown",
                        "forest green",
                        "gold",
                        "dark gold",
                        "gray",
                        "dark gray",
                        "light green",
                        "lemon chiffon",
                        "coral",
                        "sea green",
                        "sky blue",
                        "magenta",
                        "purple",
                        "slate blue",
                        "violet red",
                        "navy blue",
                        "olive",
                        "orange",
                        "red",
                        "sienna",
                        "yellow",
                    ],
                ),
                comments=dict(type="str"),
                details_level=dict(
                    type="str", choices=["uid", "standard", "full"]
                ),
                ignore_warnings=dict(type="bool"),
                ignore_errors=dict(type="bool"),
            ),
        ),
        ips=dict(type="bool"),
        ips_settings=dict(
            type="dict",
            options=dict(
                bypass_all_under_load=dict(type="bool"),
                bypass_track_method=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3",
                    ],
                ),
                activation_mode=dict(
                    type="str",
                    choices=["according-to-policy", "detect-only"],
                ),
                cpu_usage_low_threshold=dict(type="int"),
                cpu_usage_high_threshold=dict(type="int"),
                memory_usage_low_threshold=dict(type="int"),
                memory_usage_high_threshold=dict(type="int"),
                send_threat_cloud_info=dict(type="bool"),
                reject_on_cluster_fail_over=dict(type="bool"),
            ),
        ),
        members=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str"),
                interfaces=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        name=dict(type="str"),
                        anti_spoofing=dict(type="bool"),
                        anti_spoofing_settings=dict(
                            type="dict",
                            options=dict(
                                action=dict(
                                    type="str", choices=["prevent", "detect"]
                                ),
                                exclude_packets=dict(type="bool"),
                                excluded_network_name=dict(type="str"),
                                excluded_network_uid=dict(type="str"),
                                spoof_tracking=dict(
                                    type="str",
                                    choices=["none", "log", "alert"],
                                ),
                            ),
                        ),
                        ip_address=dict(type="str"),
                        ipv4_address=dict(type="str"),
                        ipv6_address=dict(type="str"),
                        network_mask=dict(type="str"),
                        ipv4_network_mask=dict(type="str"),
                        ipv6_network_mask=dict(type="str"),
                        mask_length=dict(type="str"),
                        ipv4_mask_length=dict(type="str"),
                        ipv6_mask_length=dict(type="str"),
                        security_zone=dict(type="bool"),
                        security_zone_settings=dict(
                            type="dict",
                            options=dict(
                                auto_calculated=dict(type="bool"),
                                specific_zone=dict(type="str"),
                            ),
                        ),
                        tags=dict(type="list", elements="str"),
                        topology=dict(
                            type="str",
                            choices=["automatic", "external", "internal"],
                        ),
                        topology_settings=dict(
                            type="dict",
                            options=dict(
                                interface_leads_to_dmz=dict(type="bool"),
                                ip_address_behind_this_interface=dict(
                                    type="str",
                                    choices=[
                                        "not defined",
                                        "network defined by the interface ip and net mask",
                                        "network defined by routing",
                                        "specific",
                                    ],
                                ),
                                specific_network=dict(type="str"),
                            ),
                        ),
                        color=dict(
                            type="str",
                            choices=[
                                "aquamarine",
                                "black",
                                "blue",
                                "crete blue",
                                "burlywood",
                                "cyan",
                                "dark green",
                                "khaki",
                                "orchid",
                                "dark orange",
                                "dark sea green",
                                "pink",
                                "turquoise",
                                "dark blue",
                                "firebrick",
                                "brown",
                                "forest green",
                                "gold",
                                "dark gold",
                                "gray",
                                "dark gray",
                                "light green",
                                "lemon chiffon",
                                "coral",
                                "sea green",
                                "sky blue",
                                "magenta",
                                "purple",
                                "slate blue",
                                "violet red",
                                "navy blue",
                                "olive",
                                "orange",
                                "red",
                                "sienna",
                                "yellow",
                            ],
                        ),
                        comments=dict(type="str"),
                        details_level=dict(
                            type="str", choices=["uid", "standard", "full"]
                        ),
                        ignore_warnings=dict(type="bool"),
                        ignore_errors=dict(type="bool"),
                    ),
                ),
                ip_address=dict(type="str"),
                ipv4_address=dict(type="str"),
                ipv6_address=dict(type="str"),
                one_time_password=dict(type="str", no_log=True),
                tags=dict(type="list", elements="str"),
                color=dict(
                    type="str",
                    choices=[
                        "aquamarine",
                        "black",
                        "blue",
                        "crete blue",
                        "burlywood",
                        "cyan",
                        "dark green",
                        "khaki",
                        "orchid",
                        "dark orange",
                        "dark sea green",
                        "pink",
                        "turquoise",
                        "dark blue",
                        "firebrick",
                        "brown",
                        "forest green",
                        "gold",
                        "dark gold",
                        "gray",
                        "dark gray",
                        "light green",
                        "lemon chiffon",
                        "coral",
                        "sea green",
                        "sky blue",
                        "magenta",
                        "purple",
                        "slate blue",
                        "violet red",
                        "navy blue",
                        "olive",
                        "orange",
                        "red",
                        "sienna",
                        "yellow",
                    ],
                ),
                comments=dict(type="str"),
                details_level=dict(
                    type="str", choices=["uid", "standard", "full"]
                ),
                ignore_warnings=dict(type="bool"),
                ignore_errors=dict(type="bool"),
            ),
        ),
        os_name=dict(type="str"),
        platform_portal_settings=dict(
            type="dict",
            options=dict(
                portal_web_settings=dict(
                    type="dict",
                    options=dict(
                        aliases=dict(type="list", elements="str"),
                        ip_address=dict(type="str"),
                        main_url=dict(type="str"),
                    ),
                ),
                certificate_settings=dict(
                    type="dict",
                    options=dict(
                        base64_certificate=dict(type="str"),
                        base64_password=dict(type="str", no_log=True),
                    ),
                ),
                accessibility=dict(
                    type="dict",
                    options=dict(
                        allow_access_from=dict(
                            type="str",
                            choices=[
                                "rule_base",
                                "internal_interfaces",
                                "all_interfaces",
                            ],
                        ),
                        internal_access_settings=dict(
                            type="dict",
                            options=dict(
                                undefined=dict(type="bool"),
                                dmz=dict(type="bool"),
                                vpn=dict(type="bool"),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        send_alerts_to_server=dict(type="list", elements="str"),
        send_logs_to_backup_server=dict(type="list", elements="str"),
        send_logs_to_server=dict(type="list", elements="str"),
        tags=dict(type="list", elements="str"),
        threat_emulation=dict(type="bool"),
        threat_extraction=dict(type="bool"),
        threat_prevention_mode=dict(
            type="str", choices=["autonomous", "custom"]
        ),
        url_filtering=dict(type="bool"),
        usercheck_portal_settings=dict(
            type="dict",
            options=dict(
                enabled=dict(type="bool"),
                portal_web_settings=dict(
                    type="dict",
                    options=dict(
                        aliases=dict(type="list", elements="str"),
                        ip_address=dict(type="str"),
                        main_url=dict(type="str"),
                    ),
                ),
                certificate_settings=dict(
                    type="dict",
                    options=dict(
                        base64_certificate=dict(type="str"),
                        base64_password=dict(type="str", no_log=True),
                    ),
                ),
                accessibility=dict(
                    type="dict",
                    options=dict(
                        allow_access_from=dict(
                            type="str",
                            choices=[
                                "rule_base",
                                "internal_interfaces",
                                "all_interfaces",
                            ],
                        ),
                        internal_access_settings=dict(
                            type="dict",
                            options=dict(
                                undefined=dict(type="bool"),
                                dmz=dict(type="bool"),
                                vpn=dict(type="bool"),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        cluster_version=dict(type="str"),
        vpn=dict(type="bool"),
        vpn_settings=dict(
            type="dict",
            options=dict(
                authentication=dict(
                    type="dict",
                    options=dict(
                        authentication_clients=dict(
                            type="list", elements="str"
                        )
                    ),
                ),
                link_selection=dict(
                    type="dict",
                    options=dict(
                        ip_selection=dict(
                            type="str",
                            choices=[
                                "use-main-address",
                                "use-selected-address-from-topology",
                                "use-statically-nated-ip",
                                "calculated-ip-based-on-topology",
                                "dns-resolving-from-hostname",
                                "dns-resolving-from-gateway-and-domain-name",
                                "use-probing-with-high-availability",
                                "use-probing-with-load-sharing",
                                "use-one-time-probing",
                            ],
                        ),
                        dns_resolving_hostname=dict(type="str"),
                        ip_address=dict(type="str"),
                    ),
                ),
                maximum_concurrent_ike_negotiations=dict(type="int"),
                maximum_concurrent_tunnels=dict(type="int"),
                office_mode=dict(
                    type="dict",
                    options=dict(
                        mode=dict(
                            type="str",
                            choices=["off", "specific-group", "all-users"],
                        ),
                        group=dict(type="str"),
                        allocate_ip_address_from=dict(
                            type="dict",
                            options=dict(
                                radius_server=dict(type="bool"),
                                use_allocate_method=dict(type="bool"),
                                allocate_method=dict(
                                    type="str", choices=["manual", "automatic"]
                                ),
                                manual_network=dict(type="str"),
                                dhcp_server=dict(type="str"),
                                virtual_ip_address=dict(type="str"),
                                dhcp_mac_address=dict(
                                    type="str",
                                    choices=["per-machine", "per-user"],
                                ),
                                optional_parameters=dict(
                                    type="dict",
                                    options=dict(
                                        use_primary_dns_server=dict(
                                            type="bool"
                                        ),
                                        primary_dns_server=dict(type="str"),
                                        use_first_backup_dns_server=dict(
                                            type="bool"
                                        ),
                                        first_backup_dns_server=dict(
                                            type="str"
                                        ),
                                        use_second_backup_dns_server=dict(
                                            type="bool"
                                        ),
                                        second_backup_dns_server=dict(
                                            type="str"
                                        ),
                                        dns_suffixes=dict(type="str"),
                                        use_primary_wins_server=dict(
                                            type="bool"
                                        ),
                                        primary_wins_server=dict(type="str"),
                                        use_first_backup_wins_server=dict(
                                            type="bool"
                                        ),
                                        first_backup_wins_server=dict(
                                            type="str"
                                        ),
                                        use_second_backup_wins_server=dict(
                                            type="bool"
                                        ),
                                        second_backup_wins_server=dict(
                                            type="str"
                                        ),
                                        ip_lease_duration=dict(type="int"),
                                    ),
                                ),
                            ),
                        ),
                        support_multiple_interfaces=dict(type="bool"),
                        perform_anti_spoofing=dict(type="bool"),
                        anti_spoofing_additional_addresses=dict(type="str"),
                    ),
                ),
                remote_access=dict(
                    type="dict",
                    options=dict(
                        support_l2tp=dict(type="bool"),
                        l2tp_auth_method=dict(
                            type="str", choices=["certificate", "md5"]
                        ),
                        l2tp_certificate=dict(type="str"),
                        allow_vpn_clients_to_route_traffic=dict(type="bool"),
                        support_nat_traversal_mechanism=dict(type="bool"),
                        nat_traversal_service=dict(type="str"),
                        support_visitor_mode=dict(type="bool"),
                        visitor_mode_service=dict(type="str"),
                        visitor_mode_interface=dict(type="str"),
                    ),
                ),
                vpn_domain=dict(type="str"),
                vpn_domain_type=dict(
                    type="str", choices=["manual", "addresses_behind_gw"]
                ),
            ),
        ),
        show_portals_certificate=dict(type="bool"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        groups=dict(type="list", elements="str"),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "simple-cluster"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this plugin util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os.path
import re
import typing as t

from ansible_collections.community.dns.plugins.module_utils._names import (
    InvalidDomainName,
    normalize_label,
    split_into_labels,
)

_BEGIN_SUBSET_MATCHER = re.compile(r"===BEGIN ([^=]*) DOMAINS===")
_END_SUBSET_MATCHER = re.compile(r"===END ([^=]*) DOMAINS===")


class PublicSuffixEntry:
    """
    Contains a Public Suffix List entry with metadata.
    """

    def __init__(
        self,
        labels: tuple[str, ...],
        exception_rule: bool = False,
        part: str | None = None,
    ) -> None:
        self.labels = labels
        self.exception_rule = exception_rule
        self.part = part

    def matches(self, normalized_labels) -> bool:
        """
        Match PSL entry with a given normalized list of labels.
        """
        if len(normalized_labels) < len(self.labels):
            return False
        for i, label in enumerate(self.labels):
            normalized_label = normalized_labels[i]
            if label not in (normalized_label, "*"):
                return False
        return True


def select_prevailing_rule(rules: list[PublicSuffixEntry]) -> PublicSuffixEntry:
    """
    Given a non-empty set of rules matching a domain name, finds the prevailing rule.

    It uses the algorithm specified on https://publicsuffix.org/list/.
    """
    max_length_rule = rules[0]
    max_length = len(max_length_rule.labels)
    for rule in rules:
        if rule.exception_rule:
            return rule
        if len(rule.labels) > max_length:
            max_length = len(rule.labels)
            max_length_rule = rule
    return max_length_rule


class PublicSuffixList:
    """
    Contains the Public Suffix List.
    """

    def __init__(self, rules: list[PublicSuffixEntry]) -> None:
        self._generic_rule = PublicSuffixEntry(("*",))
        self._rules = sorted(rules, key=lambda entry: entry.labels)

    @classmethod
    def load(cls, filename: str) -> t.Self:
        """
        Load Public Suffix List from the given filename.
        """
        rules: list[PublicSuffixEntry] = []
        part: str | None = None
        with open(filename, "rb") as content_file:
            content = content_file.read().decode("utf-8")
        for line in content.splitlines():
            line = line.strip()
            if line.startswith("//") or not line:
                m = _BEGIN_SUBSET_MATCHER.search(line)
                if m:
                    part = m.group(1).lower()
                m = _END_SUBSET_MATCHER.search(line)
                if m:
                    part = None
                continue
            if part is None:
                raise AssertionError("Internal error: found PSL entry with no part!")
            exception_rule = False
            if line.startswith("!"):
                exception_rule = True
                line = line[1:]
            if line.startswith("."):
                line = line[1:]
            labels = tuple(
                normalize_label(label) for label in split_into_labels(line)[0]
            )
            rules.append(
                PublicSuffixEntry(labels, exception_rule=exception_rule, part=part)
            )
        return cls(rules)

    def get_suffix_length_and_rule(
        self, normalized_labels: list[str], icann_only: bool = False
    ) -> tuple[int, PublicSuffixEntry | None]:
        """
        Given a list of normalized labels, searches for a matching rule.

        Returns the tuple ``(suffix_length, rule)``. The ``rule`` is never ``None``
        except if ``normalized_labels`` is empty, in which case ``(0, None)`` is returned.

        If ``icann_only`` is set to ``True``, only official ICANN rules are used. If
        ``icann_only`` is ``False`` (default), also private rules are used.
        """
        if not normalized_labels:
            return 0, None

        # Find matching rules
        rules: list[PublicSuffixEntry] = []
        for rule in self._rules:
            if icann_only and rule.part != "icann":
                continue
            if rule.matches(normalized_labels):
                rules.append(rule)
        if not rules:
            rules.append(self._generic_rule)

        # Select prevailing rule
        rule = select_prevailing_rule(rules)

        # Determine suffix
        suffix_length = len(rule.labels)
        if rule.exception_rule:
            suffix_length -= 1

        # Return result
        return suffix_length, rule

    def get_suffix(
        self,
        domain: str,
        keep_unknown_suffix: bool = True,
        normalize_result: bool = False,
        icann_only: bool = False,
    ) -> str:
        """
        Given a domain name, extracts the public suffix.

        If ``keep_unknown_suffix`` is set to ``False``, only suffixes matching explicit
        entries from the PSL are returned. If ``keep_unknown_suffix`` is ``True`` (default),
        the implicit ``*`` rule is used if no other rule matches.

        If ``normalize_result`` is set to ``True``, the result is re-combined form the
        normalized labels. In that case, the result is lower-case ASCII. If
        ``normalize_result`` is ``False`` (default), the result ``result`` always satisfies
        ``domain.endswith(result)``.

        If ``icann_only`` is set to ``True``, only official ICANN rules are used. If
        ``icann_only`` is ``False`` (default), also private rules are used.
        """
        # Split into labels and normalize
        try:
            labels, tail = split_into_labels(domain)
            normalized_labels = [normalize_label(label) for label in labels]
        except InvalidDomainName:
            return ""
        if normalize_result:
            labels = normalized_labels

        # Get suffix length
        suffix_length, rule = self.get_suffix_length_and_rule(
            normalized_labels, icann_only=icann_only
        )
        if rule is None:
            return ""
        if not keep_unknown_suffix and rule is self._generic_rule:
            return ""
        return ".".join(reversed(labels[:suffix_length])) + tail

    def get_registrable_domain(
        self,
        domain: str,
        keep_unknown_suffix: bool = True,
        only_if_registerable: bool = True,
        normalize_result: bool = False,
        icann_only: bool = False,
    ) -> str:
        """
        Given a domain name, extracts the registrable domain. This is the public suffix
        including the last label before the suffix.

        If ``keep_unknown_suffix`` is set to ``False``, only suffixes matching explicit
        entries from the PSL are returned. If no suffix can be found, ``''`` is returned.
        If ``keep_unknown_suffix`` is ``True`` (default), the implicit ``*`` rule is used
        if no other rule matches.

        If ``only_if_registerable`` is set to ``False``, the public suffix is returned
        if there is no label before the suffix. If ``only_if_registerable`` is ``True``
        (default), ``''`` is returned in that case.

        If ``normalize_result`` is set to ``True``, the result is re-combined form the
        normalized labels. In that case, the result is lower-case ASCII. If
        ``normalize_result`` is ``False`` (default), the result ``result`` always satisfies
        ``domain.endswith(result)``.

        If ``icann_only`` is set to ``True``, only official ICANN rules are used. If
        ``icann_only`` is ``False`` (default), also private rules are used.
        """
        # Split into labels and normalize
        try:
            labels, tail = split_into_labels(domain)
            normalized_labels = [normalize_label(label) for label in labels]
        except InvalidDomainName:
            return ""
        if normalize_result:
            labels = normalized_labels

        # Get suffix length
        suffix_length, rule = self.get_suffix_length_and_rule(
            normalized_labels, icann_only=icann_only
        )
        if rule is None:
            return ""
        if not keep_unknown_suffix and rule is self._generic_rule:
            return ""
        if suffix_length < len(labels):
            suffix_length += 1
        elif only_if_registerable:
            return ""
        return ".".join(reversed(labels[:suffix_length])) + tail


# The official Public Suffix List
PUBLIC_SUFFIX_LIST = PublicSuffixList.load(
    os.path.join(os.path.dirname(__file__), "..", "public_suffix_list.dat")
)



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._argspec import ArgumentSpec
from ansible_collections.community.dns.plugins.module_utils._hosttech.json_api import (
    HostTechJSONAPI,
)
from ansible_collections.community.dns.plugins.module_utils._hosttech.wsdl_api import (
    HostTechWSDLAPI,
)
from ansible_collections.community.dns.plugins.module_utils._provider import (
    ProviderInformation,
)
from ansible_collections.community.dns.plugins.module_utils._wsdl import HAS_LXML_ETREE
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    DNSAPIError,
)

if t.TYPE_CHECKING:
    from .._argspec import OptionProvider  # pragma: no cover
    from .._http import HTTPHelper  # pragma: no cover
    from .._provider import AnsibleType  # pragma: no cover


class HosttechProviderInformation(ProviderInformation):
    def get_supported_record_types(self) -> list[str]:
        """
        Return a list of supported record types.
        """
        return ["A", "CNAME", "MX", "AAAA", "TXT", "PTR", "SRV", "SPF", "NS", "CAA"]

    def get_zone_id_type(self) -> AnsibleType:
        """
        Return the (short) type for zone IDs, like ``'int'`` or ``'str'``.
        """
        return "int"

    def get_record_id_type(self) -> AnsibleType:
        """
        Return the (short) type for record IDs, like ``'int'`` or ``'str'``.
        """
        return "int"

    def get_record_default_ttl(self) -> int | None:
        """
        Return the default TTL for records, like 300, 3600 or None.
        None means that some other TTL (usually from the zone) will be used.
        """
        return 3600

    def normalize_prefix(self, prefix: str | None) -> str | None:
        """
        Given a prefix (string or None), return its normalized form.

        The result should always be None for the trivial prefix, and a non-zero length DNS name
        for a non-trivial prefix.

        If a provider supports other identifiers for the trivial prefix, such as '@', this
        function needs to convert them to None as well.
        """
        return prefix or None

    def txt_record_handling(
        self,
    ) -> t.Literal["decoded", "encoded", "encoded-no-char-encoding"]:
        """
        Return how the API handles TXT records.

        Returns one of the following strings:
        * 'decoded' - the API works with unencoded values
        * 'encoded' - the API works with encoded values
        * 'encoded-no-char-encoding' - the API works with encoded values, but without character encoding
        """
        return "decoded"


def create_hosttech_provider_information() -> HosttechProviderInformation:
    return HosttechProviderInformation()


def create_hosttech_argument_spec() -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "hosttech_username": {"type": "str"},
            "hosttech_password": {"type": "str", "no_log": True},
            "hosttech_token": {"type": "str", "no_log": True, "aliases": ["api_token"]},
        },
        required_together=[("hosttech_username", "hosttech_password")],
        mutually_exclusive=[("hosttech_username", "hosttech_token")],
    )


def create_hosttech_api(
    option_provider: OptionProvider, http_helper: HTTPHelper
) -> HostTechJSONAPI | HostTechWSDLAPI:
    username = option_provider.get_option("hosttech_username")
    password = option_provider.get_option("hosttech_password")
    if username is not None and password is not None:
        if not HAS_LXML_ETREE:
            raise DNSAPIError("Needs lxml Python module (pip install lxml)")

        return HostTechWSDLAPI(http_helper, username, password, debug=False)

    token = option_provider.get_option("hosttech_token")
    if token is not None:
        return HostTechJSONAPI(http_helper, token)

    raise DNSAPIError(
        "One of hosttech_token or both hosttech_username and hosttech_password must be provided!"
    )
